@@ -20,6 +20,7 @@
 
 #include <cscc/c/c_internal.h>
 #include "il_serialize.h"
+#include "il_crypt.h"
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -356,6 +357,16 @@ ILType *CTypeCreateCharPtr(ILGenInfo *info)
 	return charPtr;
 }
 
+ILType *CTypeCreateWCharPtr(ILGenInfo *info)
+{
+	static ILType *wcharPtr = 0;
+	if(!wcharPtr)
+	{
+		wcharPtr = CTypeCreatePointer(info, ILType_Char);
+	}
+	return wcharPtr;
+}
+
 ILType *CTypeAddConst(ILGenInfo *info, ILType *type)
 {
 	ILClass *classInfo;
@@ -680,7 +691,7 @@ ILType *CTypeDefineAnonEnum(ILGenInfo *info, const char *funcName)
 	sprintf(name, "(%ld)", number);
 
 	/* Create a new name by prepending "enum " to the name */
-	newName = (char *)ILMalloc(strlen(name) + funcNameLen + 1);
+	newName = (char *)ILMalloc(strlen(name) + funcNameLen + 6);
 	if(!newName)
 	{
 		ILGenOutOfMemory(info);
@@ -723,6 +734,18 @@ ILType *CTypeDefineAnonEnum(ILGenInfo *info, const char *funcName)
 	return ILType_FromValueType(classInfo);
 }
 
+ILType *CTypeResolveAnonEnum(ILType *type)
+{
+	if(!CTypeIsAnonEnum(type))
+	{
+		return type;
+	}
+	else
+	{
+		return ILTypeGetEnumType(ILTypeStripPrefixes(type));
+	}
+}
+
 ILField *CTypeDefineField(ILGenInfo *info, ILType *structType,
 					 	  const char *fieldName, ILType *fieldType)
 {
@@ -733,14 +756,34 @@ ILField *CTypeDefineField(ILGenInfo *info, ILType *structType,
 	int isUnion = (strncmp(ILClass_Name(classInfo), "union ", 6) == 0);
 	ILField *field;
 
+	/* Convert open arrays into zero-length arrays, so that
+	   "type[]" can be used in a struct to mean "type[0]" */
+	if(CTypeIsOpenArray(fieldType))
+	{
+		fieldType = CTypeCreateArray(info, CTypeGetElemType(fieldType), 0);
+	}
+
 	/* Determine the size and alignment of the new field */
 	size = CTypeSizeAndAlign(fieldType, &align);
 	if(layout)
 	{
-		if(size == CTYPE_DYNAMIC || size == CTYPE_UNKNOWN)
+		if(size == CTYPE_UNKNOWN)
 		{
 			return 0;
 		}
+		if(size == CTYPE_DYNAMIC)
+		{
+			/* Special case: allow "long double" to be used inside
+			   struct's and union's, even though it is a dynamic type */
+			if(ILTypeStripPrefixes(fieldType) == ILType_Float)
+			{
+				size = align;
+			}
+			else
+			{
+				return 0;
+			}
+		}
 	}
 	else
 	{
@@ -953,21 +996,21 @@ static ILUInt32 BitFieldLeftOver(ILGenInfo *info, ILClass *classInfo,
  * Add an attribute that defines a bit field name.
  */
 static void BitFieldAdd(ILGenInfo *info, ILClass *classInfo,
-						const char *fieldName, const char *storageName,
+						const char *fieldName,
+						unsigned long fieldLen,
+						const char *storageName,
+						unsigned long storageLen,
 						ILUInt32 posn, ILUInt32 numBits)
 {
 	unsigned char fieldHeader[IL_META_COMPRESS_MAX_SIZE];
 	unsigned char storageHeader[IL_META_COMPRESS_MAX_SIZE];
-	unsigned long fieldLen, storageLen;
 	int fieldHeaderLen, storageHeaderLen;
 	unsigned long totalLen;
 	unsigned char *buf;
 	ILAttribute *attr;
 
 	/* Determine the total length of the attribute value */
-	fieldLen = strlen(fieldName);
 	fieldHeaderLen = ILMetaCompressData(fieldHeader, fieldLen);
-	storageLen = strlen(storageName);
 	storageHeaderLen = ILMetaCompressData(storageHeader, storageLen);
 	totalLen = 12 + fieldHeaderLen + fieldLen + storageHeaderLen + storageLen;
 
@@ -1078,15 +1121,381 @@ int CTypeDefineBitField(ILGenInfo *info, ILType *structType,
 		{
 			posn -= numBits;
 		}
-		BitFieldAdd(info, classInfo, fieldName,
-					ILField_Name(field), posn, numBits);
+		BitFieldAdd(info, classInfo, fieldName, strlen(fieldName),
+					ILField_Name(field), strlen(ILField_Name(field)),
+					posn, numBits);
 	}
 
 	/* Done */
 	return 1;
 }
 
-void CTypeEndStruct(ILGenInfo *info, ILType *structType)
+/*
+ * Add a character to an MD5 context.
+ */
+#define	MD5HashAddChar(md5,value)	\
+			do { \
+				unsigned char ch = (unsigned char)(value); \
+				ILMD5Data((md5), &ch, 1); \
+			} while (0)
+
+/*
+ * Add a 32-bit size value to an MD5 context.
+ */
+static void MD5HashAddSize(ILMD5Context *md5, ILUInt32 value)
+{
+	unsigned char buf[4];
+	IL_WRITE_UINT32(buf, value);
+	ILMD5Data(md5, buf, 4);
+}
+
+/*
+ * Add a class name to an MD5 context.
+ */
+static void MD5HashAddName(ILMD5Context *md5, ILClass *classInfo)
+{
+	const char *namespace = ILClass_Namespace(classInfo);
+	const char *name = ILClass_Name(classInfo);
+	if(namespace)
+	{
+		ILMD5Data(md5, namespace, strlen(namespace));
+		MD5HashAddChar(md5, '.');
+	}
+	ILMD5Data(md5, name, strlen(name) + 1);
+}
+
+/*
+ * Forward reference.
+ */
+static void MD5HashFields(ILMD5Context *md5, ILClass *classInfo);
+
+/*
+ * Hash a type using a given MD5 context.
+ */
+static void MD5HashType(ILMD5Context *md5, ILType *type)
+{
+	if(ILType_IsPrimitive(type))
+	{
+		/* Hash a primitive type */
+		MD5HashAddChar(md5, ILType_ToElement(type));
+	}
+	else if(ILType_IsValueType(type))
+	{
+		int structKind = CTypeGetStructKind(type);
+		if(structKind != -1)
+		{
+			/* Hash a struct or union type */
+			MD5HashAddChar(md5, structKind + 100);
+			MD5HashAddName(md5, ILType_ToValueType(type));
+		}
+		else if(CTypeIsOpenArray(type))
+		{
+			/* Hash an open array type */
+			MD5HashAddChar(md5, 110);
+			MD5HashAddSize(md5, 0);
+			MD5HashType(md5, CTypeGetElemType(type));
+		}
+		else if(CTypeIsArray(type))
+		{
+			MD5HashAddChar(md5, 110);
+			MD5HashAddSize(md5, CTypeGetNumElems(type));
+			MD5HashType(md5, CTypeGetElemType(type));
+		}
+		else
+		{
+			/* Hash an ordinary value type */
+			MD5HashAddChar(md5, IL_META_ELEMTYPE_VALUETYPE);
+			MD5HashAddName(md5, ILType_ToValueType(type));
+		}
+	}
+	else if(ILType_IsClass(type))
+	{
+		/* Hash an object reference type */
+		MD5HashAddChar(md5, IL_META_ELEMTYPE_CLASS);
+		MD5HashAddName(md5, ILType_ToClass(type));
+	}
+	else if(type != 0 && ILType_IsComplex(type))
+	{
+		/* Hash a complex type */
+		switch(ILType_Kind(type))
+		{
+			case IL_TYPE_COMPLEX_BYREF:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_BYREF);
+				MD5HashType(md5, ILType_Ref(type));
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_PTR:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_PTR);
+				MD5HashType(md5, ILType_Ref(type));
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_ARRAY:
+			case IL_TYPE_COMPLEX_ARRAY_CONTINUE:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_ARRAY);
+				MD5HashAddSize(md5, (ILUInt32)(ILTypeGetRank(type)));
+				MD5HashType(md5, ILTypeGetElemType(type));
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_CMOD_REQD:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_CMOD_REQD);
+				MD5HashAddName(md5, type->un.modifier__.info__);
+				MD5HashType(md5, type->un.modifier__.type__);
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_CMOD_OPT:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_CMOD_OPT);
+				MD5HashAddName(md5, type->un.modifier__.info__);
+				MD5HashType(md5, type->un.modifier__.type__);
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_SENTINEL:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_SENTINEL);
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_PINNED:
+			{
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_PINNED);
+				MD5HashType(md5, ILType_Ref(type));
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_METHOD:
+			case IL_TYPE_COMPLEX_METHOD | IL_TYPE_COMPLEX_METHOD_SENTINEL:
+			{
+				unsigned long numParams;
+				unsigned long param;
+				MD5HashAddChar(md5, IL_META_ELEMTYPE_FNPTR);
+				MD5HashType(md5, ILTypeGetReturnWithPrefixes(type));
+				numParams = ILTypeNumParams(type);
+				MD5HashAddSize(md5, (ILUInt32)numParams);
+				for(param = 1; param <= numParams; ++param)
+				{
+					MD5HashType(md5, ILTypeGetParamWithPrefixes(type, param));
+				}
+			}
+			break;
+		}
+	}
+}
+
+/*
+ * Hash the fields within a struct or union type.
+ */
+static void MD5HashFields(ILMD5Context *md5, ILClass *classInfo)
+{
+	ILField *field = 0;
+	const char *name;
+	while((field = (ILField *)ILClassNextMemberByKind
+				(classInfo, (ILMember *)field, IL_META_MEMBERKIND_FIELD)) != 0)
+	{
+		if(!ILField_IsStatic(field))
+		{
+			name = ILField_Name(field);
+			ILMD5Data(md5, name, strlen(name) + 1);
+			MD5HashType(md5, ILField_Type(field));
+		}
+	}
+}
+
+/*
+ * Create a new name for a top-level anonymous struct or union.
+ */
+static char *CreateNewAnonName(ILGenInfo *info, ILType *type,
+							   ILClass *classInfo)
+{
+	ILMD5Context md5;
+	unsigned char hash[IL_MD5_HASH_SIZE];
+	int structKind, posn;
+	char name[64];
+	int value, index, bits;
+	static char const encode[] =
+		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_$";
+	char *newName;
+
+	/* Hash the fields in the structure */
+	ILMD5Init(&md5);
+	MD5HashFields(&md5, classInfo);
+	ILMD5Finalize(&md5, hash);
+
+	/* Format the name */
+	structKind = CTypeGetStructKind(type);
+	if(structKind == C_STKIND_STRUCT || structKind == C_STKIND_STRUCT_NATIVE)
+	{
+		strcpy(name, "struct (");
+		posn = 8;
+	}
+	else
+	{
+		strcpy(name, "union (");
+		posn = 7;
+	}
+	value = 0;
+	bits = 0;
+	for(index = 0; index < IL_MD5_HASH_SIZE; ++index)
+	{
+		value = (value << 8) + hash[index];
+		bits += 8;
+		while(bits >= 6)
+		{
+			bits -= 6;
+			name[posn++] = encode[value >> bits];
+			value &= ((1 << bits) - 1);
+		}
+	}
+	value <<= (6 - bits);
+	name[posn++] = encode[value >> bits];
+	name[posn++] = ')';
+	name[posn++] = '\0';
+
+	/* Duplicate the string and return */
+	newName = ILDupString(name);
+	if(!newName)
+	{
+		ILGenOutOfMemory(info);
+	}
+	return newName;
+}
+
+/*
+ * Clone the contents of a structure type.
+ */
+static void CloneStruct(ILGenInfo *info, ILClass *dest, ILClass *src)
+{
+	ILField *field;
+	ILField *newField;
+	ILFieldLayout *flayout;
+	ILClassLayout *clayout;
+	ILMethod *ctor;
+	ILAttribute *attr;
+	const void *blob;
+	unsigned long blobLen;
+	ILSerializeReader *reader;
+	const char *str;
+	int slen;
+	const char *fieldName;
+	unsigned long fieldLen;
+	const char *storageName;
+	unsigned long storageLen;
+	ILUInt32 bitFieldStart;
+	ILUInt32 bitFieldSize;
+
+	/* Clone the fields */
+	field = 0;
+	while((field = (ILField *)ILClassNextMemberByKind
+				(src, (ILMember *)field, IL_META_MEMBERKIND_FIELD)) != 0)
+	{
+		newField = ILFieldCreate(dest, 0, ILField_Name(field),
+								 ILField_Attrs(field));
+		if(!newField)
+		{
+			ILGenOutOfMemory(info);
+		}
+		ILMemberSetSignature
+			((ILMember *)newField, ILFieldGetTypeWithPrefixes(field));
+		flayout = ILFieldLayoutGetFromOwner(field);
+		if(flayout)
+		{
+			if(!ILFieldLayoutCreate(info->image, 0, newField,
+									ILFieldLayoutGetOffset(flayout)))
+			{
+				ILGenOutOfMemory(info);
+			}
+		}
+	}
+
+	/* Clone the bit field declarations */
+	ctor = BitFieldCtor(info);
+	attr = 0;
+	while((attr = ILProgramItemNextAttribute(ILToProgramItem(src), attr)) != 0)
+	{
+		/* Skip this attribute if it is not "BitFieldAttribute" */
+		if(ILAttributeTypeAsItem(attr) != ILToProgramItem(ctor))
+		{
+			continue;
+		}
+
+		/* Extract the bit field definition's parameters */
+		blob = ILAttributeGetValue(attr, &blobLen);
+		if(!blob)
+		{
+			continue;
+		}
+		reader = ILSerializeReaderInit(ctor, blob, blobLen);
+		if(!reader)
+		{
+			continue;
+		}
+		if(ILSerializeReaderGetParamType(reader) != IL_META_SERIALTYPE_STRING)
+		{
+			ILSerializeReaderDestroy(reader);
+			continue;
+		}
+		if((slen = ILSerializeReaderGetString(reader, &str)) < 0)
+		{
+			ILSerializeReaderDestroy(reader);
+			continue;
+		}
+		fieldName = str;
+		fieldLen = (unsigned long)(long)slen;
+		if(ILSerializeReaderGetParamType(reader) != IL_META_SERIALTYPE_STRING)
+		{
+			ILSerializeReaderDestroy(reader);
+			continue;
+		}
+		if((slen = ILSerializeReaderGetString(reader, &str)) < 0)
+		{
+			ILSerializeReaderDestroy(reader);
+			continue;
+		}
+		storageName = str;
+		storageLen = (unsigned long)(long)slen;
+		if(ILSerializeReaderGetParamType(reader) != IL_META_SERIALTYPE_I4)
+		{
+			ILSerializeReaderDestroy(reader);
+			continue;
+		}
+		bitFieldStart = (ILUInt32)(ILSerializeReaderGetInt32
+			(reader, IL_META_SERIALTYPE_I4));
+		if(ILSerializeReaderGetParamType(reader) != IL_META_SERIALTYPE_I4)
+		{
+			ILSerializeReaderDestroy(reader);
+			continue;
+		}
+		bitFieldSize = (ILUInt32)(ILSerializeReaderGetInt32
+			(reader, IL_META_SERIALTYPE_I4));
+		ILSerializeReaderDestroy(reader);
+
+		/* Create a new bit field declaration on the cloned copy */
+		BitFieldAdd(info, dest, fieldName, fieldLen,
+					storageName, storageLen, bitFieldStart, bitFieldSize);
+	}
+
+	/* Clone the class layout information */
+	clayout = ILClassLayoutGetFromOwner(src);
+	if(clayout)
+	{
+		if(!ILClassLayoutCreate(info->image, 0, dest,
+								ILClassLayoutGetPackingSize(clayout),
+								ILClassLayoutGetClassSize(clayout)))
+		{
+			ILGenOutOfMemory(info);
+		}
+	}
+}
+
+ILType *CTypeEndStruct(ILGenInfo *info, ILType *structType, int renameAnon)
 {
 	ILClass *classInfo = ILType_ToValueType(structType);
 	ILClassLayout *layout = ILClassLayoutGetFromOwner(classInfo);
@@ -1101,6 +1510,35 @@ void CTypeEndStruct(ILGenInfo *info, ILType *structType)
 			ILClassLayoutSetClassSize(layout, size);
 		}
 	}
+	if(renameAnon)
+	{
+		char *newName;
+		ILClass *newClass;
+
+		/* Create a new name for the anonymous struct or union */
+		newName = CreateNewAnonName(info, structType, classInfo);
+
+		/* See if we already have a type defined with this name */
+		newClass = ILClassLookup(ILClassGlobalScope(info->image), newName, 0);
+		if(newClass)
+		{
+			ILFree(newName);
+			return ILType_FromValueType(newClass);
+		}
+
+		/* Create a new type and clone the original structure into it */
+		newClass = ILClassCreate(ILClassGlobalScope(info->image), 0,
+							     newName, 0, ILClass_ParentRef(classInfo));
+		if(!newClass)
+		{
+			ILGenOutOfMemory(info);
+		}
+		ILFree(newName);
+		ILClassSetAttrs(newClass, ~0, ILClass_Attrs(classInfo));
+		CloneStruct(info, newClass, classInfo);
+		return ILType_FromValueType(newClass);
+	}
+	return structType;
 }
 
 ILField *CTypeLookupField(ILGenInfo *info, ILType *structType,
@@ -1406,6 +1844,19 @@ int CTypeIsEnum(ILType *type)
 	return 0;
 }
 
+int CTypeIsAnonEnum(ILType *type)
+{
+	type = ILTypeStripPrefixes(type);
+	if(ILType_IsValueType(type))
+	{
+		if(!strncmp(ILClass_Name(ILType_ToValueType(type)), "enum (", 6))
+		{
+			return 1;
+		}
+	}
+	return 0;
+}
+
 int CTypeIsArray(ILType *type)
 {
 	type = ILTypeStripPrefixes(type);
@@ -1634,7 +2085,44 @@ ILType *CTypeGetElemType(ILType *type)
 	field = FindArrayElemField(classInfo);
 	if(field)
 	{
-		return ILField_Type(field);
+		type = ILFieldGetTypeWithPrefixes(field);
+		if(CTypeIsFunctionPtr(type))
+		{
+			return type;
+		}
+		else
+		{
+			return ILField_Type(field);
+		}
+	}
+	else
+	{
+		return 0;
+	}
+}
+
+ILUInt32 CTypeGetNumFields(ILType *type)
+{
+	ILClass *classInfo;
+	ILField *field;
+	ILUInt32 num;
+
+	type = ILTypeStripPrefixes(type);
+	if(CTypeIsStruct(type) || CTypeIsUnion(type))
+	{
+		classInfo = ILType_ToValueType(type);
+		field = 0;
+		num = 0;
+		while((field = (ILField *)ILClassNextMemberByKind
+					(classInfo, (ILMember *)field,
+					 IL_META_MEMBERKIND_FIELD)) != 0)
+		{
+			if(!ILField_IsStatic(field))
+			{
+				++num;
+			}
+		}
+		return num;
 	}
 	else
 	{
