@@ -114,18 +114,30 @@ static int IsAttributeUsage(ILClass *classInfo)
 }
 
 /*
- * Modify an attribute name so that it ends in "Attribute".
+ * Modify an attribute name node so that it uses ILNode_AttrQualIdent
+ * or ILNode_AttrIdentifier for the top-level attribute lookup.
  */
-static void ModifyAttrName(ILNode_Identifier *ident,int force)
+static ILNode *ModifyAttrName(ILNode *node)
 {
-	char *name = ident->name;
-	int namelen = strlen(name);
-	if(force || (namelen < 9 || strcmp(name + namelen - 9, "Attribute") != 0))
+	ILNode *newNode;
+	if(yyisa(node, ILNode_QualIdent))
 	{
-		ident->name = ILInternAppendedString
-			(ILInternString(name, namelen),
-			 ILInternString("Attribute", 9)).string;
+		newNode = ILNode_AttrQualIdent_create
+			(((ILNode_QualIdent *)node)->left,
+			 ((ILNode_QualIdent *)node)->right);
 	}
+	else if(yyisa(node, ILNode_Identifier))
+	{
+		newNode = ILNode_AttrIdentifier_create
+			(((ILNode_Identifier *)node)->name);
+	}
+	else
+	{
+		return node;
+	}
+	yysetfilename(newNode, yygetfilename(node));
+	yysetlinenum(newNode, yygetlinenum(node));
+	return newNode;
 }
 
 /*
@@ -143,7 +155,8 @@ static ILProgramItem *LookupAttrField(ILGenInfo *info, ILType *type,
 	while(classInfo != 0)
 	{
 		member = 0;
-		while((member = ILClassNextMember(classInfo, member)) != 0)
+		while((member = ILClassNextMemberMatch
+				(classInfo, member, 0, name, 0)) != 0)
 		{
 			/* Skip members that aren't accessible to the module */
 			if(!ILMemberAccessible(member, scope))
@@ -151,12 +164,6 @@ static ILProgramItem *LookupAttrField(ILGenInfo *info, ILType *type,
 				continue;
 			}
 
-			/* Check the member name */
-			if(strcmp(ILMember_Name(member), name) != 0)
-			{
-				continue;
-			}
-
 			/* Return the field or property */
 			if(ILMember_IsField(member))
 			{
@@ -209,6 +216,215 @@ static ILType *GetAttrFieldType(ILProgramItem *item)
 	}
 }
 
+/*
+ * Check if there are any circular semantic analysis cases .
+ * The attribute target might not be distinct from the attribute
+ * itself , like being a nested class or a member of the attribute.
+ */
+static int IsAttributeTargetDistinct(ILGenInfo *info, ILProgramItem *item,
+					ILClass *classInfo)
+{
+	ILClass *target;
+	if(ILToProgramItem(classInfo)==item)return 0;
+	target= ILProgramItemToClass(item);
+	if(target==NULL)
+	{
+		ILMember *member=ILProgramItemToMember(item);
+		if(member)
+		{
+			target=ILMember_Owner(member);
+		}
+	}
+	while(target!=NULL)
+	{
+		if(ILToProgramItem(classInfo)==ILToProgramItem(target))
+		{
+			return 0;
+		}
+		target=ILClass_NestedParent(target);
+	}
+	return 1;
+}
+
+/*
+ * Convert a type into a name, formatted for use in attribute values.
+ */
+static const char *CSTypeToAttrName(ILGenInfo *info, ILType *type)
+{
+	ILClass *classInfo = ILTypeToClass(info, type);
+	const char *name = ILClass_Name(classInfo);
+	const char *namespace = ILClass_Namespace(classInfo);
+	const char *finalName;
+	if(namespace)
+	{
+		finalName = ILInternAppendedString
+					(ILInternAppendedString
+						(ILInternString((char *)namespace, -1),
+						 ILInternString((char *)".", 1)),
+					 ILInternString((char *)name, -1)).string;
+	}
+	else
+	{
+		finalName = name;
+	}
+	if(ILClass_NestedParent(classInfo) != 0)
+	{
+		/* Prepend the name of the enclosing nesting class */
+		const char *parentName = CSTypeToAttrName
+			(info, ILType_FromClass(ILClass_NestedParent(classInfo)));
+		finalName = ILInternAppendedString
+					(ILInternAppendedString
+						(ILInternString((char *)parentName, -1),
+						 ILInternString((char *)"+", 1)),
+					 ILInternString((char *)finalName, -1)).string;
+	}
+	return finalName;
+}
+
+/* 
+ * write an entry into the serialized stream using the provide paramType and
+ * argValue and serialType. 
+ */
+
+static void WriteSerializedEntry(ILGenInfo *info,
+								 ILSerializeWriter *writer, 
+								 ILType *paramType,
+								 ILEvalValue *argValue,
+								 ILType *argType,
+								 int serialType)
+{	
+	ILType *systemType=ILFindSystemType(info,"Type");
+
+	switch(serialType)
+	{
+		case IL_META_SERIALTYPE_BOOLEAN:
+		case IL_META_SERIALTYPE_I1:
+		case IL_META_SERIALTYPE_U1:
+		case IL_META_SERIALTYPE_I2:
+		case IL_META_SERIALTYPE_U2:
+		case IL_META_SERIALTYPE_CHAR:
+		case IL_META_SERIALTYPE_I4:
+		case IL_META_SERIALTYPE_U4:
+		{
+			ILSerializeWriterSetInt32(writer, argValue->un.i4Value,
+									  serialType);
+		}
+		break;
+
+		case IL_META_SERIALTYPE_I8:
+		case IL_META_SERIALTYPE_U8:
+		{
+			ILSerializeWriterSetInt64(writer, argValue->un.i8Value);
+		}
+		break;
+
+		case IL_META_SERIALTYPE_R4:
+		{
+			ILSerializeWriterSetFloat32(writer, argValue->un.r4Value);
+		}
+		break;
+
+		case IL_META_SERIALTYPE_R8:
+		{
+			ILSerializeWriterSetFloat64(writer, argValue->un.r8Value);
+		}
+		break;
+
+		case IL_META_SERIALTYPE_STRING:
+		{
+			ILSerializeWriterSetString(writer, argValue->un.strValue.str,
+									   argValue->un.strValue.len);
+		}
+		break;
+
+		case IL_META_SERIALTYPE_TYPE:
+		{
+			const char *name = CSTypeToAttrName
+				(info, (ILType *)(argValue->un.strValue.str));
+			ILSerializeWriterSetString(writer, name, strlen(name));
+		}
+		break;
+
+		case IL_META_SERIALTYPE_VARIANT:
+		{
+			/* Note : We assume the values are castable and
+			 * do not provide any checks here */
+			if(ILType_IsPrimitive(argType))
+			{
+				switch(argValue->valueType)
+				{	
+					case ILMachineType_Boolean:
+					case ILMachineType_Int8:
+					case ILMachineType_UInt8:
+					case ILMachineType_Int16:
+					case ILMachineType_UInt16:
+					case ILMachineType_Char:
+					case ILMachineType_Int32:
+					case ILMachineType_UInt32:
+					case ILMachineType_Int64:
+					case ILMachineType_UInt64:
+					case ILMachineType_Float32:
+					case ILMachineType_Float64:
+					case ILMachineType_Decimal:
+					{
+						serialType=ILSerializeGetType(argType);
+						
+						ILSerializeWriterSetBoxedPrefix(writer, 
+														   serialType);
+
+						WriteSerializedEntry(info, writer, paramType, 
+											 argValue, argType, serialType);
+					}
+					break;
+					
+					case ILMachineType_String:
+					{
+						/* TODO */
+					}
+					break;
+					
+					default:
+					{
+					}
+					break;
+				}
+			}
+			else if(ILTypeIdentical(argType, systemType))
+			{ 
+				ILSerializeWriterSetBoxedPrefix(writer,
+										IL_META_SERIALTYPE_TYPE);
+				
+				WriteSerializedEntry(info, writer, paramType, 
+									 argValue, argType,
+									 IL_META_SERIALTYPE_TYPE);
+			}
+			else if(ILTypeIsEnum(argType))
+			{
+				const char *name = CSTypeToAttrName(info, (ILType *)(argType));
+				ILSerializeWriterSetBoxedPrefix(writer,
+										IL_META_SERIALTYPE_ENUM);
+				ILSerializeWriterSetString(writer, name, strlen(name));
+
+				serialType=ILSerializeGetType(argType);
+
+				WriteSerializedEntry(info, writer, paramType,
+										argValue, argType,
+										serialType);	
+			}
+		}
+		break;
+
+		default:
+		{
+			if(ILType_IsArray(paramType))
+			{
+				/* TODO: arrays */
+			}
+		}
+		break;
+	}
+}
+
 /*
  * Process a single attribute in a section.
  */
@@ -244,63 +460,14 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 	const void *blob;
 	unsigned long blobLen;
 	ILAttribute *attribute;
-	ILNode *nameNode=NULL;
-	int alreadySeen=0;
+	int skipConst;
+	ILType *argType;
 
-	/* Make sure that the attribute type name ends in "Attribute" */
-	if(yyisa(attr->name, ILNode_Identifier))
-	{
-		ModifyAttrName((ILNode_Identifier *)(attr->name),0);
-	}
-	else if(yyisa(attr->name, ILNode_QualIdent))
-	{
-		ModifyAttrName
-		  ((ILNode_Identifier *)(((ILNode_QualIdent *)(attr->name))->right),0);
-	}
-
-	nameNode=attr->name;
-	if(!CSSemExpectType(attr->name,info,&(attr->name)))
-	{
-		/* restore and rename */
-		attr->name=nameNode;
-		/* Make sure that the attribute type name ends in "Attribute" */
-		if(yyisa(attr->name, ILNode_Identifier))
-		{
-			ModifyAttrName((ILNode_Identifier *)(attr->name),1);
-		}
-		else if(yyisa(attr->name, ILNode_QualIdent))
-		{
-			ModifyAttrName((ILNode_Identifier *)(((ILNode_QualIdent *)
-											(attr->name))->right),1);
-		}
-		alreadySeen=1;
-	}
-	
-	/* Perform semantic analysis on the attribute type */
+	/* Modify the name so as to correctly handle "Attribute" suffixes,
+	   and then perform semantic analysis on the type */
+	attr->name = ModifyAttrName(attr->name);
 	type = CSSemType(attr->name, info, &(attr->name));
 
-	/* To handle stuff like System.Xml.XmlAttribute which is not an
-	 * attribute, but there is System.Xml.Serialization.XmlAttributeAttribute
-	 * which is also [XmlAttribute]
-	 */
-	if((!alreadySeen) && (!ILTypeAssignCompatible
-			(info->image, type, ILFindSystemType(info, "Attribute"))))
-	{
-		attr->name=nameNode;
-		/* Make sure that the attribute type name ends in "Attribute" */
-		if(yyisa(attr->name, ILNode_Identifier))
-		{
-			ModifyAttrName((ILNode_Identifier *)(attr->name),1);
-		}
-		else if(yyisa(attr->name, ILNode_QualIdent))
-		{
-			ModifyAttrName((ILNode_Identifier *)(((ILNode_QualIdent *)
-											(attr->name))->right),1);
-		}
-		/* Perform semantic analysis on the attribute type */
-		type = CSSemType(attr->name, info, &(attr->name));
-	}
-
 	/* The type must inherit from "System.Attribute" and not be abstract */
 	if(!ILTypeAssignCompatible
 			(info->image, type, ILFindSystemType(info, "Attribute")))
@@ -324,8 +491,12 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 	   circularities in the semantic analysis network */
 	if(!IsAttributeUsage(classInfo))
 	{
-		/* Perform semantic analysis on the attribute type */
-		CSSemProgramItem(info, ILToProgramItem(classInfo));
+		/* Perform semantic analysis on the attribute type, but only
+		   if we aren't trying to apply the attribute to itself */
+		if(IsAttributeTargetDistinct(info,item,classInfo))
+		{
+			CSSemProgramItem(info, ILToProgramItem(classInfo));
+		}
 
 		/* Get the usage information for the attribute */
 		allowMultiple = 1;
@@ -389,8 +560,11 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 			   have to worry about reporting errors here */
 			if(!CSSemExpectValue(arg, info, iter.last, &value))
 			{
-				haveErrors = 1;
-				evalArgs[argNum].type = ILType_Int32;
+				if(!CSSemIsType(value))
+				{
+					haveErrors = 1;
+					evalArgs[argNum].type = ILType_Int32;
+				}
 			}
 			else
 			{
@@ -401,10 +575,19 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 			evalArgs[argNum].modifier = ILParamMod_empty;
 
 			/* Evaluate the constant value of the argument */
-			if(!haveErrors &&
-			   !ILNode_EvalConst(*(iter.last), info, &(evalValues[argNum])))
+			if(CSSemIsType(value))
 			{
-				haveErrors = 1;
+				evalValues[argNum].valueType = ILMachineType_Void;
+				evalValues[argNum].un.oValue = CSSemGetType(value);
+				evalArgs[argNum].type = ILFindSystemType(info, "Type");
+			}
+			else
+			{
+				if(!haveErrors &&
+				   !ILNode_EvalConst(*(iter.last), info, &(evalValues[argNum])))
+				{
+					haveErrors = 1;
+				}
 			}
 
 			/* Advance to the next argument */
@@ -468,8 +651,11 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 			if(!CSSemExpectValue(((ILNode_NamedArg *)arg)->value, info,
 								 &(((ILNode_NamedArg *)arg)->value), &value))
 			{
-				haveErrors = 1;
-				namedArgs[argNum].type = ILType_Int32;
+				if(!CSSemIsType(value))
+				{
+					haveErrors = 1;
+					namedArgs[argNum].type = ILType_Int32;
+				}
 			}
 			else
 			{
@@ -480,11 +666,22 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 			namedArgs[argNum].modifier = ILParamMod_empty;
 
 			/* Evaluate the constant value of the argument */
-			if(!haveErrors &&
-			   !ILNode_EvalConst(namedArgs[argNum].node, info,
-			   					 &(namedValues[argNum])))
+			if(CSSemIsType(value))
 			{
-				haveErrors = 1;
+				namedValues[argNum].valueType = ILMachineType_Void;
+				namedValues[argNum].un.oValue = CSSemGetType(value);
+				namedArgs[argNum].type = ILFindSystemType(info, "Type");
+				skipConst = 1;
+			}
+			else
+			{
+				if(!haveErrors &&
+				   !ILNode_EvalConst(namedArgs[argNum].node, info,
+				   					 &(namedValues[argNum])))
+				{
+					haveErrors = 1;
+				}
+				skipConst = 0;
 			}
 
 			/* Cast the constant to the final type */
@@ -494,11 +691,11 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 							namedArgs[argNum].parent,
 							namedArgs[argNum].type,
 							GetAttrFieldType(namedFields[argNum]),1) &&
-				   ILGenCastConst
+				   (skipConst || ILGenCastConst
 				   		(info, &(namedValues[argNum]),
 				   		 namedValues[argNum].valueType,
 						 ILTypeToMachineType
-								(GetAttrFieldType(namedFields[argNum]))))
+								(GetAttrFieldType(namedFields[argNum])))))
 				{
 					namedArgs[argNum].node = *(namedArgs[argNum].parent);
 				}
@@ -596,23 +793,28 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 	for(argNum = 0; argNum < numArgs; ++argNum)
 	{
 		paramType = ILTypeGetParam(signature, argNum + 1);
-		if(!ILGenCastConst(info, &(evalValues[argNum]),
-						   evalValues[argNum].valueType,
-						   ILTypeToMachineType(paramType)))
+		if(evalValues[argNum].valueType != ILMachineType_Void)
 		{
-			CCErrorOnLine(yygetfilename(evalArgs[argNum].node),
-						  yygetlinenum(evalArgs[argNum].node),
-						  _("could not coerce constant argument %d"),
-						  argNum + 1);
-			haveErrors = 1;
-		}
-		else if(ILSerializeGetType(paramType) == -1)
-		{
-			CCErrorOnLine(yygetfilename(evalArgs[argNum].node),
-						  yygetlinenum(evalArgs[argNum].node),
-						  _("attribute argument %d is not serializable"),
-						  argNum + 1);
-			haveErrors = 1;
+			if(!ILGenCastConst(info, &(evalValues[argNum]),
+				   evalValues[argNum].valueType,ILTypeToMachineType(paramType))
+			   && !ILCanCastKind(info, evalArgs[argNum].type,
+								paramType, IL_CONVERT_STANDARD,0))
+			{
+								
+				CCErrorOnLine(yygetfilename(evalArgs[argNum].node),
+							  yygetlinenum(evalArgs[argNum].node),
+							  _("could not coerce constant argument %d"),
+							  argNum + 1);
+				haveErrors = 1;
+			}
+			else if(ILSerializeGetType(paramType) == -1)
+			{
+				CCErrorOnLine(yygetfilename(evalArgs[argNum].node),
+							  yygetlinenum(evalArgs[argNum].node),
+							  _("attribute argument %d is not serializable"),
+							  argNum + 1);
+				haveErrors = 1;
+			}
 		}
 	}
 	if(haveErrors)
@@ -629,63 +831,25 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 	for(argNum = 0; argNum < numArgs; ++argNum)
 	{
 		paramType = ILTypeGetParam(signature, argNum + 1);
-		serialType = ILSerializeGetType(paramType);
 		argValue = &(evalValues[argNum]);
-		switch(serialType)
-		{
-			case IL_META_SERIALTYPE_BOOLEAN:
-			case IL_META_SERIALTYPE_I1:
-			case IL_META_SERIALTYPE_U1:
-			case IL_META_SERIALTYPE_I2:
-			case IL_META_SERIALTYPE_U2:
-			case IL_META_SERIALTYPE_CHAR:
-			case IL_META_SERIALTYPE_I4:
-			case IL_META_SERIALTYPE_U4:
-			{
-				ILSerializeWriterSetInt32(writer, argValue->un.i4Value,
-										  serialType);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_I8:
-			case IL_META_SERIALTYPE_U8:
-			{
-				ILSerializeWriterSetInt64(writer, argValue->un.i8Value);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_R4:
-			{
-				ILSerializeWriterSetFloat32(writer, argValue->un.r4Value);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_R8:
-			{
-				ILSerializeWriterSetFloat64(writer, argValue->un.r8Value);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_STRING:
-			{
-				ILSerializeWriterSetString(writer, argValue->un.strValue.str,
-										   argValue->un.strValue.len);
-			}
-			break;
-
-			default:
-			{
-				/* TODO: types and arrays */
-			}
-			break;
-		}
+		argType = evalArgs[argNum].type;
+		serialType = ILSerializeGetType(paramType);
+		WriteSerializedEntry(info,writer,paramType,argValue,argType,serialType);
 	}
 	ILSerializeWriterSetNumExtra(writer, numNamedArgs);
 	for(argNum = 0; argNum < numNamedArgs; ++argNum)
 	{
 		argValue = &(namedValues[argNum]);
-		paramType = ILValueTypeToType(info, argValue->valueType);
-		serialType = ILSerializeGetType(paramType);
+		if(argValue->valueType == ILMachineType_Void)
+		{
+			serialType = IL_META_SERIALTYPE_TYPE;
+			paramType = NULL;
+		}
+		else
+		{
+			paramType = ILValueTypeToType(info, argValue->valueType);
+			serialType = ILSerializeGetType(paramType);
+		}
 		if(ILMember_IsField(((ILMember *)(namedFields[argNum]))))
 		{
 			ILSerializeWriterSetField
@@ -698,54 +862,8 @@ static void ProcessAttr(ILGenInfo *info, ILProgramItem *item,
 				(writer, ILMember_Name((ILMember *)(namedFields[argNum])),
 				 serialType);
 		}
-		switch(serialType)
-		{
-			case IL_META_SERIALTYPE_BOOLEAN:
-			case IL_META_SERIALTYPE_I1:
-			case IL_META_SERIALTYPE_U1:
-			case IL_META_SERIALTYPE_I2:
-			case IL_META_SERIALTYPE_U2:
-			case IL_META_SERIALTYPE_CHAR:
-			case IL_META_SERIALTYPE_I4:
-			case IL_META_SERIALTYPE_U4:
-			{
-				ILSerializeWriterSetInt32(writer, argValue->un.i4Value,
-										  serialType);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_I8:
-			case IL_META_SERIALTYPE_U8:
-			{
-				ILSerializeWriterSetInt64(writer, argValue->un.i8Value);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_R4:
-			{
-				ILSerializeWriterSetFloat32(writer, argValue->un.r4Value);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_R8:
-			{
-				ILSerializeWriterSetFloat64(writer, argValue->un.r8Value);
-			}
-			break;
-
-			case IL_META_SERIALTYPE_STRING:
-			{
-				ILSerializeWriterSetString(writer, argValue->un.strValue.str,
-										   argValue->un.strValue.len);
-			}
-			break;
-
-			default:
-			{
-				/* TODO: types and arrays */
-			}
-			break;
-		}
+		argType=namedArgs[argNum].type;
+		WriteSerializedEntry(info,writer,paramType,argValue,argType,serialType);
 	}
 	blob = ILSerializeWriterGetBlob(writer, &blobLen);
 	if(!blob)
@@ -819,6 +937,12 @@ void CSProcessAttrs(ILGenInfo *info, ILProgramItem *mainItem,
 	while((section = (ILNode_AttributeSection *)
 				ILNode_ListIter_Next(&iter)) != 0)
 	{
+		/* Skip documentation comments, if present */
+		if(yyisa(section, ILNode_DocComment))
+		{
+			continue;
+		}
+
 		/* Resolve the target item */
 		item = mainItem;
 		target = mainTarget;
