@@ -164,7 +164,14 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 			case IL_TYPE_COMPLEX_BYREF:
 			{
 				ILDumpType(stream, image, ILType_Ref(type), flags);
-				fputs(" &", stream);
+				if((flags & IL_DUMP_XML_QUOTING) != 0)
+				{
+					fputs(" &amp;", stream);
+				}
+				else
+				{
+					fputs(" &", stream);
+				}
 			}
 			break;
 
@@ -276,6 +283,40 @@ void ILDumpType(FILE *stream, ILImage *image, ILType *type, int flags)
 			}
 			break;
 
+			case IL_TYPE_COMPLEX_WITH:
+			{
+				unsigned long numParams;
+				unsigned long param;
+				ILDumpType(stream, image,
+						   ILTypeGetWithMainWithPrefixes(type), flags);
+				putc('<', stream);
+				numParams = ILTypeNumWithParams(type);
+				for(param = 1; param <= numParams; ++param)
+				{
+					if(param != 1)
+					{
+						fputs(", ", stream);
+					}
+					ILDumpType(stream, image,
+							   ILTypeGetWithParamWithPrefixes(type, param),
+							   flags);
+				}
+				putc('>', stream);
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_MVAR:
+			{
+				fprintf(stream, "!!%d", ILType_VarNum(type));
+			}
+			break;
+
+			case IL_TYPE_COMPLEX_VAR:
+			{
+				fprintf(stream, "!%d", ILType_VarNum(type));
+			}
+			break;
+
 			default:
 			{
 				if((type->kind__ & IL_TYPE_COMPLEX_METHOD) != 0)
@@ -359,7 +400,7 @@ static void DumpParamType(FILE *stream, ILImage *image,
 			if(type)
 			{
 				fputs(" marshal(", stream);
-				ILDumpNativeType(stream, type, typeLen);
+				ILDumpNativeType(stream, type, typeLen, flags);
 				putc(')', stream);
 			}
 		}
@@ -449,12 +490,31 @@ static void DumpParams(FILE *stream, ILImage *image, ILType *type,
 	}
 }
 
-void ILDumpMethodType(FILE *stream, ILImage *image, ILType *type, int flags,
-					  ILClass *info, const char *methodName,
-					  ILMethod *methodInfo)
+/*
+ * Internal version of "ILDumpMethodType" that can also handle
+ * instantiations of generic method calls.
+ */
+static void DumpMethodType(FILE *stream, ILImage *image, ILType *type,
+						   int flags, ILClass *info, const char *methodName,
+					  	   ILMethod *methodInfo, ILType *withTypes)
 {
 	ILUInt32 callingConventions;
 	ILType *synType;
+	int dumpGenerics;
+	ILUInt32 genericNum;
+	ILGenericPar *genPar;
+	const char *name;
+	ILProgramItem *constraint;
+	ILTypeSpec *spec;
+	unsigned long numWithParams;
+	unsigned long withParam;
+
+	/* Determine if we need to dump the generic parameters */
+	dumpGenerics = ((flags & IL_DUMP_GENERIC_PARAMS) != 0);
+
+	/* Strip off the "generic parameters" flag so that we don't
+	   end up passing it down to the parameter types */
+	flags &= ~IL_DUMP_GENERIC_PARAMS;
 
 	/* Dump the calling conventions for the method */
 	callingConventions = ILType_CallConv(type);
@@ -490,16 +550,110 @@ void ILDumpMethodType(FILE *stream, ILImage *image, ILType *type, int flags,
 		putc('*', stream);
 	}
 
+	/* Dump the generic method parameters if necessary */
+	if(dumpGenerics && methodInfo)
+	{
+		genericNum = 0;
+		genPar = ILGenericParGetFromOwner
+				(ILToProgramItem(methodInfo), genericNum);
+		if(genPar)
+		{
+			putc('<', stream);
+			do
+			{
+				if(genericNum > 0)
+				{
+					fputs(", ", stream);
+				}
+				constraint = ILGenericPar_Constraint(genPar);
+				if(constraint)
+				{
+					putc('(', stream);
+					spec = ILProgramItemToTypeSpec(constraint);
+					if(spec)
+					{
+						ILDumpType(stream, image,
+								   ILTypeSpec_Type(spec), flags);
+					}
+					else
+					{
+						ILDumpType(stream, image,
+							   	   ILClassToType((ILClass *)constraint), flags);
+					}
+					putc(')', stream);
+				}
+				name = ILGenericPar_Name(genPar);
+				if(name)
+				{
+					ILDumpIdentifier(stream, name, 0, flags);
+				}
+				else
+				{
+					fprintf(stream, "G_%d", (int)(genericNum + 1));
+				}
+				++genericNum;
+				genPar = ILGenericParGetFromOwner
+					(ILToProgramItem(methodInfo), genericNum);
+			}
+			while(genPar != 0);
+			putc('>', stream);
+		}
+	}
+	else if(withTypes)
+	{
+		/* Dump the instantiation types from a method specification */
+		putc('<', stream);
+		numWithParams = ILTypeNumParams(withTypes);
+		for(withParam = 1; withParam <= numWithParams; ++withParam)
+		{
+			if(withParam != 1)
+			{
+				fputs(", ", stream);
+			}
+			ILDumpType(stream, image,
+					   ILTypeGetParam(withTypes, withParam), flags);
+		}
+		putc('>', stream);
+	}
+
 	/* Dump the parameters */
 	putc('(', stream);
 	DumpParams(stream, image, type, methodInfo, flags);
 	putc(')', stream);
 }
 
+void ILDumpMethodType(FILE *stream, ILImage *image, ILType *type, int flags,
+					  ILClass *info, const char *methodName,
+					  ILMethod *methodInfo)
+{
+	DumpMethodType(stream, image, type, flags, info,
+				   methodName, methodInfo, 0);
+}
+
+void ILDumpMethodSpec(FILE *stream, ILImage *image,
+					  ILMethodSpec *spec, int flags)
+{
+	ILMember *member;
+	ILMethod *method;
+
+	/* Extract the member and make sure it is a method */
+	member = ILMemberResolve(ILMethodSpec_Method(spec));
+	if(!ILMember_IsMethod(member))
+	{
+		return;
+	}
+	method = (ILMethod *)member;
+
+	/* Dump the method information, together with the instantiation types */
+	DumpMethodType(stream, image, ILMethod_Signature(method), flags,
+				   ILMethod_Owner(method), ILMethod_Name(method),
+				   0, ILMethodSpec_Type(spec));
+}
+
 /*
  * Dump a variant type.
  */
-static void DumpVariantType(FILE *stream, ILUInt32 type)
+static void DumpVariantType(FILE *stream, ILUInt32 type, int flags)
 {
 	switch(type & IL_META_VARIANTTYPE_BASE_TYPE_MASK)
 	{
@@ -765,14 +919,21 @@ static void DumpVariantType(FILE *stream, ILUInt32 type)
 	}
 	if((type & IL_META_VARIANTTYPE_BYREF) != 0)
 	{
-		fputs(" &", stream);
+		if((flags & IL_DUMP_XML_QUOTING) != 0)
+		{
+			fputs(" &amp;", stream);
+		}
+		else
+		{
+			fputs(" &", stream);
+		}
 	}
 }
 
 /*
  * Inner version of "ILDumpNativeType".
  */
-static void DumpNativeType(FILE *stream, ILMetaDataRead *reader)
+static void DumpNativeType(FILE *stream, ILMetaDataRead *reader, int flags)
 {
 	unsigned long value;
 	unsigned long value2;
@@ -876,7 +1037,7 @@ static void DumpNativeType(FILE *stream, ILMetaDataRead *reader)
 
 		case IL_META_NATIVETYPE_PTR:
 		{
-			DumpNativeType(stream, reader);
+			DumpNativeType(stream, reader, flags);
 			fputs(" *", stream);
 		}
 		break;
@@ -959,7 +1120,7 @@ static void DumpNativeType(FILE *stream, ILMetaDataRead *reader)
 		{
 			fputs("safearray ", stream);
 			value = ILMetaUncompressData(reader);
-			DumpVariantType(stream, (ILUInt32)value);
+			DumpVariantType(stream, (ILUInt32)value, flags);
 		}
 		break;
 
@@ -976,7 +1137,7 @@ static void DumpNativeType(FILE *stream, ILMetaDataRead *reader)
 			if(reader->len)
 			{
 				fputs("fixed array ", stream);
-				DumpNativeType(stream, reader);
+				DumpNativeType(stream, reader, flags);
 				fprintf(stream, "[%lu]", value);
 			}
 			else
@@ -1054,7 +1215,7 @@ static void DumpNativeType(FILE *stream, ILMetaDataRead *reader)
 
 		case IL_META_NATIVETYPE_ARRAY:
 		{
-			DumpNativeType(stream, reader);
+			DumpNativeType(stream, reader, flags);
 			putc('[', stream);
 			value = ILMetaUncompressData(reader);
 			if(value)
@@ -1175,13 +1336,14 @@ static void DumpNativeType(FILE *stream, ILMetaDataRead *reader)
 	}
 }
 
-void ILDumpNativeType(FILE *stream, const void *type, unsigned long len)
+void ILDumpNativeType(FILE *stream, const void *type,
+					  unsigned long len, int flags)
 {
 	ILMetaDataRead reader;
 	reader.data = (const unsigned char *)type;
 	reader.len = len;
 	reader.error = 0;
-	DumpNativeType(stream, &reader);
+	DumpNativeType(stream, &reader, flags);
 }
 
 #ifdef	__cplusplus
