@@ -1074,6 +1074,7 @@ static ILObject *CreateResourceStream(ILExecThread *thread, ILImage *image,
 	unsigned char *section;
 	unsigned long sectionLen;
 	unsigned long start;
+	unsigned long pad;
 	ILUInt32 length;
 
 	/* Find the resource section within the image */
@@ -1096,9 +1097,17 @@ static ILObject *CreateResourceStream(ILExecThread *thread, ILImage *image,
 		{
 			return 0;
 		}
-		start += length + 4;
-		section += length + 4;
-		sectionLen -= length + 4;
+		if((length % 4) != 0)
+		{
+			pad = 4 - (length % 4);
+		}
+		else
+		{
+			pad = 0;
+		}
+		start += length + pad + 4;
+		section += length + pad + 4;
+		sectionLen -= length + pad + 4;
 		--posn;
 	}
 
@@ -2205,10 +2214,10 @@ ILObject* UnpackConstant(ILExecThread *thread,ILConstant* constant,
 }
 
 /*
- * public override Object GetValue(Object obj);
+ * internal Object GetValueInternal(Object obj);
  */
-ILObject *_IL_ClrField_GetValue(ILExecThread *thread, ILObject *_this,
-								ILObject *obj)
+ILObject *_IL_ClrField_GetValueInternal(ILExecThread *thread, ILObject *_this,
+										ILObject *obj)
 {
 	ILField *field;
 	ILType *type;
@@ -2256,8 +2265,18 @@ ILObject *_IL_ClrField_GetValue(ILExecThread *thread, ILObject *_this,
 	}
 	else if(ILField_IsStatic(field))
 	{
-		/* TODO */
-		return 0;
+		/* Get the field's type and a pointer to it */
+		type = ILField_Type(field);
+		ptr = ((ILClassPrivate *)((ILField_Owner(field))->userData))
+					->staticData;
+		if(ptr)
+		{
+			ptr = (void *)(((unsigned char *)ptr) + field->offset);
+		}
+		else
+		{
+			return 0;
+		}
 	}
 	else
 	{
@@ -2285,14 +2304,14 @@ ILObject *_IL_ClrField_GetValue(ILExecThread *thread, ILObject *_this,
 }
 
 /*
- * public override void SetValue(Object obj, Object value,
- *                               BindingFlags invokeAttr,
- *                               Binder binder, CultureInfo culture);
+ * internal void SetValueInternal(Object obj, Object value,
+ *                                BindingFlags invokeAttr,
+ *                                Binder binder, CultureInfo culture);
  */
-void _IL_ClrField_SetValue(ILExecThread *thread, ILObject *_this,
-						   ILObject *obj, ILObject *value,
-						   ILInt32 invokeAttr, ILObject *binder,
-						   ILObject *culture)
+void _IL_ClrField_SetValueInternal(ILExecThread *thread, ILObject *_this,
+						   		   ILObject *obj, ILObject *value,
+						   		   ILInt32 invokeAttr, ILObject *binder,
+						   		   ILObject *culture)
 {
 	ILField *field;
 	ILType *type;
@@ -2321,8 +2340,18 @@ void _IL_ClrField_SetValue(ILExecThread *thread, ILObject *_this,
 	}
 	else if(ILField_IsStatic(field))
 	{
-		/* TODO */
-		return;
+		/* Get the field's type and a pointer to it */
+		type = ILField_Type(field);
+		ptr = ((ILClassPrivate *)((ILField_Owner(field))->userData))
+					->staticData;
+		if(ptr)
+		{
+			ptr = (void *)(((unsigned char *)ptr) + field->offset);
+		}
+		else
+		{
+			return;
+		}
 	}
 	else
 	{
@@ -2816,12 +2845,8 @@ ILObject *_IL_ClrMethod_Invoke(ILExecThread *thread,
 	}
 	else
 	{
-		/* Static method: we must not have a target object */
-		if(obj)
-		{
-			ThrowTargetException(thread);
-			return 0;
-		}
+		/* Static method: we don't need the target object */
+		obj = 0;
 	}
 
 	/* Invoke the method */
