@@ -62,7 +62,7 @@ An example of a signature:
 		public System.String ToString();
 	}
 
-	"System.Decimal" "ToString" "(T)OSystem.String;"
+	"System.Decimal" "ToString" "(T)oSystem.String;"
 */
 
 #include "engine.h"
@@ -290,7 +290,8 @@ ILType *ILExecThreadLookupType(ILExecThread *thread, const char *typeName)
 			type = ILExecThreadLookupType(thread, typeName);
 			if(type != ILType_Invalid)
 			{
-				return ILTypeCreateArray(thread->process->context, 1, type);
+				return ILTypeFindOrCreateArray
+					(thread->process->context, 1, type);
 			}
 		}
 		break;
@@ -310,7 +311,8 @@ ILType *ILExecThreadLookupType(ILExecThread *thread, const char *typeName)
 			type = ILExecThreadLookupType(thread, typeName + 1);
 			if(type != ILType_Invalid)
 			{
-				return ILTypeCreateArray(thread->process->context, len, type);
+				return ILTypeFindOrCreateArray
+					(thread->process->context, len, type);
 			}
 		}
 		break;
@@ -699,6 +701,16 @@ ILMethod *ILExecThreadLookupMethod(ILExecThread *thread,
 								   const char *signature)
 {
 	ILClass *classInfo;
+	classInfo = ILExecThreadLookupClass(thread, typeName);
+	return ILExecThreadLookupMethodInClass
+			(thread, classInfo, methodName, signature);
+}
+
+ILMethod *ILExecThreadLookupMethodInClass(ILExecThread *thread,
+										  ILClass *classInfo,
+								   		  const char *methodName,
+								   		  const char *signature)
+{
 	ILMethod *method;
 	ILType *methodSignature;
 
@@ -708,9 +720,6 @@ ILMethod *ILExecThreadLookupMethod(ILExecThread *thread,
 		return 0;
 	}
 
-	/* Look up the type */
-	classInfo = ILExecThreadLookupClass(thread, typeName);
-
 	/* Resolve the method within the type or any of its ancestors */
 	while(classInfo != 0)
 	{
