@@ -212,7 +212,7 @@ ILProgramItem *item;
 
 #define	IsCPPointer(type,typeInfo,classInfo)	\
 			((((type) == ILEngineType_M || (type) == ILEngineType_T) && \
-			  ILTypeIdentical(typeInfo, ILType_FromValueType(classInfo))) || \
+			  ILTypeIdentical(typeInfo, ILClassToType(classInfo))) || \
 			 (unsafeAllowed && \
 			  ((type) == ILEngineType_I || (type) == ILEngineType_I4)))
 
@@ -523,7 +523,7 @@ case IL_OP_LDFLDA:
 			   pointer always returns an unmanaged pointer */
 			stack[stackSize - 1].engineType = ILEngineType_I;
 			stack[stackSize - 1].typeInfo = 0;
-			continue;
+			break;
 		}
 		else
 		{
