@@ -24,6 +24,102 @@
 extern	"C" {
 #endif
 
+ILClassName *_ILClassNameCreate(ILImage *image, ILToken token,
+								const char *name, const char *namespace,
+								ILProgramItem *scopeItem,
+								ILClassName *scopeName)
+{
+	ILClassName *className;
+
+	/* Allocate memory for the name information */
+	className = ILMemStackAlloc(&(image->memStack), ILClassName);
+	if(!className)
+	{
+		return 0;
+	}
+
+	/* Populate the name information */
+	className->image = image;
+	className->token = token;
+	className->name = _ILContextPersistString(image, name);
+	if(!(className->name))
+	{
+		return 0;
+	}
+	if(namespace)
+	{
+		className->namespace = _ILContextPersistString(image, namespace);
+		if(!(className->namespace))
+		{
+			return 0;
+		}
+	}
+	className->scope = scopeItem;
+	className->scopeName = scopeName;
+
+	/* Add the class name to the class name hash */
+	if(!ILHashAdd(image->context->classHash, className))
+	{
+		return 0;
+	}
+
+	/* Add the namespace to the context's namespace table if not present */
+	if(namespace)
+	{
+		if(ILHashFind(image->context->namespaceHash, namespace) == 0)
+		{
+			if(!ILHashAdd(image->context->namespaceHash, className))
+			{
+				return 0;
+			}
+		}
+	}
+
+	/* Done */
+	return className;
+}
+
+void _ILClassNameUpdateToken(ILClass *info)
+{
+	info->className->token = info->programItem.token;
+}
+
+ILClass *_ILClassNameToClass(ILClassName *className)
+{
+	if(className)
+	{
+		return ILClass_FromToken(className->image, className->token);
+	}
+	else
+	{
+		return 0;
+	}
+}
+
+ILClassName *_ILClassNameLookup(ILImage *image, ILProgramItem *scopeItem,
+								ILClassName *scopeName, const char *name,
+								const char *namespace)
+{
+	ILClassKeyInfo key;
+	ILClassName *className;
+	key.name = name;
+	key.nameLen = strlen(name);
+	key.namespace = namespace;
+	key.namespaceLen = (namespace ? strlen(namespace) : 0);
+	key.scopeItem = scopeItem;
+	key.scopeName = scopeName;
+	key.image = 0;
+	key.wantGlobal = 0;
+	key.ignoreCase = 0;
+	className = ILHashFindType(image->context->classHash, &key, ILClassName);
+	if(className && !(className->scope) && scopeItem)
+	{
+		/* Update the scope item now that we know what it should be */
+		className->scope = scopeItem;
+	}
+	return className;
+}
+
 /*
  * Add a nesting relationship between two classes.
  */
@@ -132,6 +228,8 @@ static ILClass *CreateClass(ILImage *image, const char *name,
 							ILProgramItem *scope)
 {
 	ILClass *info;
+	ILClassName *scopeName;
+	ILClass *scopeClass;
 
 	/* Allocate space for the class information block */
 	info = ILMemStackAlloc(&(image->memStack), ILClass);
@@ -143,9 +241,32 @@ static ILClass *CreateClass(ILImage *image, const char *name,
 	/* Initialize the class fields */
 	info->programItem.image = image;
 	info->parent = parent;
-	info->scope = scope;
 	info->ext = 0;
 
+	/* Get the name of the scope if it is a nesting parent */
+	scopeClass = ILProgramItemToClass(scope);
+	if(scopeClass)
+	{
+		scopeName = scopeClass->className;
+	}
+	else
+	{
+		scopeName = 0;
+	}
+
+	/* Create the class name record */
+	info->className = _ILClassNameLookup
+			(image, scope, scopeName, name, namespace);
+	if(!(info->className))
+	{
+		info->className = _ILClassNameCreate
+				(image, 0, name, namespace, scope, scopeName);
+		if(!(info->className))
+		{
+			return 0;
+		}
+	}
+
 	/* Create a nesting relationship with the scope if necessary */
 	if(ILClassIsNestingScope(scope))
 	{
@@ -155,27 +276,6 @@ static ILClass *CreateClass(ILImage *image, const char *name,
 		}
 	}
 
-	/* Create persistent versions of the name and namespace */
-	info->name = _ILContextPersistString(image, name);
-	if(!(info->name))
-	{
-		return 0;
-	}
-	if(namespace)
-	{
-		info->namespace = _ILContextPersistString(image, namespace);
-		if(!(info->namespace))
-		{
-			return 0;
-		}
-	}
-
-	/* Add the class to the context's hash table */
-	if(!ILHashAdd(image->context->classHash, info))
-	{
-		return 0;
-	}
-
 	/* Done */
 	return info;
 }
@@ -212,19 +312,21 @@ ILClass *ILClassCreate(ILProgramItem *scope, ILToken token, const char *name,
 			info->implements = 0;
 
 			/* Set the token code for the class */
+			info->programItem.token = token;
 			if(!_ILImageSetToken(image, &(info->programItem),
 								 token, IL_META_TOKEN_TYPE_DEF))
 			{
 				return 0;
 			}
+			_ILClassNameUpdateToken(info);
 
 			/* If this is a nested class, then move it to the
 			   end of the nesting list so that the fields and
 			   methods will appear in the correct order in the
 			   final output file */
-			if(ILClassIsNestingScope(info->scope))
+			if(ILClassIsNestingScope(info->className->scope))
 			{
-				MoveNestedToEnd((ILClass *)(info->scope), info);
+				MoveNestedToEnd((ILClass *)(info->className->scope), info);
 			}
 
 			/* Return the class to the caller */
@@ -247,6 +349,7 @@ ILClass *ILClassCreate(ILProgramItem *scope, ILToken token, const char *name,
 		{
 			return 0;
 		}
+		_ILClassNameUpdateToken(info);
 	}
 	return info;
 }
@@ -274,6 +377,7 @@ ILClass *ILClassCreateRef(ILProgramItem *scope, ILToken token,
 		{
 			return 0;
 		}
+		_ILClassNameUpdateToken(info);
 	}
 
 	/* Ready to go */
@@ -288,7 +392,7 @@ ILClass *ILClassResolve(ILClass *info)
 ILProgramItem *ILClassGlobalScope(ILImage *image)
 {
 	/* The global scope is the first module definition record */
-	if(image->tokenCount[IL_META_TOKEN_MODULE >> 24] > 0)
+	if(image->tokenCount[((ILUInt32)IL_META_TOKEN_MODULE) >> 24] > 0)
 	{
 		return (ILProgramItem *)ILImageTokenInfo
 					(image, (IL_META_TOKEN_MODULE | 1));
@@ -302,7 +406,8 @@ ILProgramItem *ILClassGlobalScope(ILImage *image)
 int ILClassIsNestingScope(ILProgramItem *scope)
 {
 	if((scope->token & IL_META_TOKEN_MASK) == IL_META_TOKEN_TYPE_REF ||
-	   (scope->token & IL_META_TOKEN_MASK) == IL_META_TOKEN_TYPE_DEF)
+	   (scope->token & IL_META_TOKEN_MASK) == IL_META_TOKEN_TYPE_DEF ||
+	   (scope->token & IL_META_TOKEN_MASK) == IL_META_TOKEN_EXPORTED_TYPE)
 	{
 		return 1;
 	}
@@ -314,6 +419,7 @@ int ILClassIsNestingScope(ILProgramItem *scope)
 
 ILClass *ILClassImport(ILImage *image, ILClass *info)
 {
+	ILClassName *newName;
 	ILClass *newInfo;
 	ILProgramItem *scope;
 	ILClassKeyInfo key;
@@ -324,11 +430,19 @@ ILClass *ILClassImport(ILImage *image, ILClass *info)
 		return info;
 	}
 
+	/* Do we have a link from the class back to the required image? */
+	newInfo = (ILClass *)_ILProgramItemLinkedBackTo
+							((ILProgramItem *)info, image);
+	if(newInfo)
+	{
+		return newInfo;
+	}
+
 	/* Determine the scope to use for the import */
-	if(ILClassIsNestingScope(info->scope))
+	if(ILClassIsNestingScope(info->className->scope))
 	{
 		/* Importing a nested class, so import its parent first */
-		newInfo = ILClassImport(image, (ILClass *)(info->scope));
+		newInfo = ILClassImport(image, (ILClass *)(info->className->scope));
 		if(!newInfo)
 		{
 			return 0;
@@ -347,16 +461,25 @@ ILClass *ILClassImport(ILImage *image, ILClass *info)
 	}
 
 	/* See if we already have a reference to this class */
-	key.name = info->name;
-	key.nameLen = strlen(info->name);
-	key.namespace = info->namespace;
-	key.namespaceLen = (info->namespace ? strlen(info->namespace) : 0);
-	key.scope = scope;
+	key.name = info->className->name;
+	key.nameLen = strlen(info->className->name);
+	key.namespace = info->className->namespace;
+	key.namespaceLen = (info->className->namespace ?
+							strlen(info->className->namespace) : 0);
+	key.scopeItem = scope;
+	key.scopeName = 0;
 	key.image = image;
 	key.wantGlobal = 0;
-	newInfo = ILHashFindType(image->context->classHash, &key, ILClass);
-	if(newInfo != 0)
+	newName = ILHashFindType(image->context->classHash, &key, ILClassName);
+	if(newName != 0)
 	{
+		/* Resolve the named class */
+		newInfo = _ILClassNameToClass(newName);
+		if(!(newInfo))
+		{
+			return 0;
+		}
+
 		/* Link "newInfo" to "info" */
 		if(!_ILProgramItemLinkedTo(&(newInfo->programItem)))
 		{
@@ -370,7 +493,8 @@ ILClass *ILClassImport(ILImage *image, ILClass *info)
 	}
 
 	/* Create a new reference */
-	newInfo = ILClassCreateRef(scope, 0, info->name, info->namespace);
+	newInfo = ILClassCreateRef(scope, 0, info->className->name,
+							   info->className->namespace);
 	if(!newInfo)
 	{
 		return 0;
@@ -404,6 +528,20 @@ ILClass *ILClassGetParent(ILClass *info)
 	}
 }
 
+void ILClassSetParent(ILClass *info, ILClass *parent)
+{
+	ILImage *image;
+
+	if(!info || !parent || ILClassIsComplete(info))
+	{
+		return;
+	}
+
+	image = ILClassToImage(parent);
+	parent = ILClassImport(image, parent);
+	info->parent = parent;
+}
+
 ILClass *ILClassGetParentRef(ILClass *info)
 {
 	if(info->parent)
@@ -429,17 +567,17 @@ void ILClassSetAttrs(ILClass *info, ILUInt32 mask, ILUInt32 values)
 
 ILProgramItem *ILClassGetScope(ILClass *info)
 {
-	return info->scope;
+	return info->className->scope;
 }
 
 const char *ILClassGetName(ILClass *info)
 {
-	return info->name;
+	return info->className->name;
 }
 
 const char *ILClassGetNamespace(ILClass *info)
 {
-	return info->namespace;
+	return info->className->namespace;
 }
 
 ILType *ILClassGetSynType(ILClass *info)
@@ -460,15 +598,17 @@ ILContext *ILClassToContext(ILClass *info)
 /*
  * Matching function that is used by "_ILClassRemoveAllFromHash".
  */
-static int ClassRemove_Match(const ILClass *classInfo, ILImage *key)
+static int ClassRemove_Match(const ILClassName *className, ILImage *key)
 {
-	return (classInfo->programItem.image == key);
+	return (className->image == key);
 }
 
 void _ILClassRemoveAllFromHash(ILImage *image)
 {
 	ILHashRemoveSubset(image->context->classHash,
 					   (ILHashMatchFunc)ClassRemove_Match, image, 0);
+	ILHashRemoveSubset(image->context->namespaceHash,
+					   (ILHashMatchFunc)ClassRemove_Match, image, 0);
 }
 
 int ILClassIsValid(ILClass *info)
@@ -507,11 +647,13 @@ ILClass *ILClassLookup(ILProgramItem *scope,
 	key.nameLen = strlen(name);
 	key.namespace = namespace;
 	key.namespaceLen = (namespace ? strlen(namespace) : 0);
-	key.scope = scope;
+	key.scopeItem = scope;
+	key.scopeName = 0;
 	key.image = 0;
 	key.wantGlobal = 0;
 	key.ignoreCase = 0;
-	return ILHashFindType(scope->image->context->classHash, &key, ILClass);
+	return _ILClassNameToClass
+		(ILHashFindType(scope->image->context->classHash, &key, ILClassName));
 }
 
 ILClass *ILClassLookupLen(ILProgramItem *scope,
@@ -523,11 +665,13 @@ ILClass *ILClassLookupLen(ILProgramItem *scope,
 	key.nameLen = nameLen;
 	key.namespace = namespace;
 	key.namespaceLen = namespaceLen;
-	key.scope = scope;
+	key.scopeItem = scope;
+	key.scopeName = 0;
 	key.image = 0;
 	key.wantGlobal = 0;
 	key.ignoreCase = 0;
-	return ILHashFindType(scope->image->context->classHash, &key, ILClass);
+	return _ILClassNameToClass
+		(ILHashFindType(scope->image->context->classHash, &key, ILClassName));
 }
 
 /*
@@ -617,7 +761,8 @@ static int CompareUnicode(const char *str,
 /*
  * Match a hash table element against a supplied key.
  */
-static int UnicodeMatch(const ILClass *classInfo, const ILClassKeyInfo *key)
+static int UnicodeMatch(const ILClassName *classInfo,
+						const ILClassKeyInfo *key)
 {
 	/* Match the namespace */
 	if(classInfo->namespace)
@@ -642,13 +787,17 @@ static int UnicodeMatch(const ILClass *classInfo, const ILClassKeyInfo *key)
 	}
 
 	/* Match the scope */
-	if(key->scope && key->scope != classInfo->scope)
+	if(key->scopeItem && key->scopeItem != classInfo->scope)
+	{
+		return 0;
+	}
+	else if(key->scopeName && key->scopeName != classInfo->scopeName)
 	{
 		return 0;
 	}
 
 	/* Match the image */
-	if(key->image && key->image != classInfo->programItem.image)
+	if(key->image && key->image != classInfo->image)
 	{
 		return 0;
 	}
@@ -656,9 +805,15 @@ static int UnicodeMatch(const ILClass *classInfo, const ILClassKeyInfo *key)
 	/* Do we only want types at the global level? */
 	if(key->wantGlobal)
 	{
+		if(classInfo->scopeName)
+		{
+			/* Nested type */
+			return 0;
+		}
 		if((classInfo->scope->token & IL_META_TOKEN_MASK) !=
 					IL_META_TOKEN_MODULE)
 		{
+			/* Imported type */
 			return 0;
 		}
 	}
@@ -677,13 +832,15 @@ ILClass *ILClassLookupUnicode(ILProgramItem *scope,
 	key.nameLen = nameLen;
 	key.namespace = (const char *)namespace;
 	key.namespaceLen = namespaceLen;
-	key.scope = scope;
+	key.scopeItem = scope;
+	key.scopeName = 0;
 	key.image = 0;
 	key.wantGlobal = 0;
 	key.ignoreCase = ignoreCase;
-	return ILHashFindAltType(scope->image->context->classHash, &key, ILClass,
-							 (ILHashKeyComputeFunc)UnicodeHashKey,
-							 (ILHashMatchFunc)UnicodeMatch);
+	return _ILClassNameToClass
+		(ILHashFindAltType(scope->image->context->classHash, &key, ILClassName,
+						   (ILHashKeyComputeFunc)UnicodeHashKey,
+						   (ILHashMatchFunc)UnicodeMatch));
 }
 
 ILClass *ILClassLookupGlobal(ILContext *context,
@@ -694,11 +851,13 @@ ILClass *ILClassLookupGlobal(ILContext *context,
 	key.nameLen = strlen(name);
 	key.namespace = namespace;
 	key.namespaceLen = (namespace ? strlen(namespace) : 0);
-	key.scope = 0;
+	key.scopeItem = 0;
+	key.scopeName = 0;
 	key.image = 0;
 	key.wantGlobal = 1;
 	key.ignoreCase = 0;
-	return ILHashFindType(context->classHash, &key, ILClass);
+	return _ILClassNameToClass
+		(ILHashFindType(context->classHash, &key, ILClassName));
 }
 
 ILClass *ILClassLookupGlobalLen(ILContext *context,
@@ -710,11 +869,13 @@ ILClass *ILClassLookupGlobalLen(ILContext *context,
 	key.nameLen = nameLen;
 	key.namespace = namespace;
 	key.namespaceLen = namespaceLen;
-	key.scope = 0;
+	key.scopeItem = 0;
+	key.scopeName = 0;
 	key.image = 0;
 	key.wantGlobal = 1;
 	key.ignoreCase = 0;
-	return ILHashFindType(context->classHash, &key, ILClass);
+	return _ILClassNameToClass
+		(ILHashFindType(context->classHash, &key, ILClassName));
 }
 
 ILClass *ILClassLookupGlobalUnicode(ILContext *context,
@@ -727,13 +888,15 @@ ILClass *ILClassLookupGlobalUnicode(ILContext *context,
 	key.nameLen = nameLen;
 	key.namespace = (const char *)namespace;
 	key.namespaceLen = namespaceLen;
-	key.scope = 0;
+	key.scopeItem = 0;
+	key.scopeName = 0;
 	key.image = 0;
 	key.wantGlobal = 1;
 	key.ignoreCase = ignoreCase;
-	return ILHashFindAltType(context->classHash, &key, ILClass,
-							 (ILHashKeyComputeFunc)UnicodeHashKey,
-							 (ILHashMatchFunc)UnicodeMatch);
+	return _ILClassNameToClass
+		(ILHashFindAltType(context->classHash, &key, ILClassName,
+						   (ILHashKeyComputeFunc)UnicodeHashKey,
+						   (ILHashMatchFunc)UnicodeMatch));
 }
 
 ILImplements *ILClassAddImplements(ILClass *info, ILClass *interface,
@@ -964,9 +1127,9 @@ int ILClassCanNest(ILClass *parent, ILClass *child)
 
 ILClass *ILClassGetNestedParent(ILClass *info)
 {
-	if(ILClassIsNestingScope(info->scope))
+	if(ILClassIsNestingScope(info->className->scope))
 	{
-		return (ILClass *)(info->scope);
+		return (ILClass *)(info->className->scope);
 	}
 	else
 	{
@@ -1002,7 +1165,7 @@ static int AccessibleNestedPrivate(ILClass *info, ILClass *scope)
 	{
 		return 0;
 	}
-	if(info->scope == (ILProgramItem *)scope)
+	if(info->className->scope == (ILProgramItem *)scope)
 	{
 		return 1;
 	}
@@ -1120,6 +1283,7 @@ int ILClassAccessible(ILClass *info, ILClass *scope)
 
 ILMember *ILClassNextMember(ILClass *info, ILMember *last)
 {
+	info = ILClassGetUnderlying(info);
 	if(last)
 	{
 		return last->nextMember;
@@ -1136,14 +1300,65 @@ ILMember *ILClassNextMember(ILClass *info, ILMember *last)
 
 ILMember *ILClassNextMemberByKind(ILClass *info, ILMember *last, int kind)
 {
-	while((last = ILClassNextMember(info, last)) != 0)
+	info = ILClassGetUnderlying(info);
+	if(last)
+	{
+		last = last->nextMember;
+	}
+	else if(info)
+	{
+		last = info->firstMember;
+	}
+	else
+	{
+		return 0;
+	}
+	while(last != 0)
 	{
 		if(((int)(last->kind)) == kind)
 		{
-			break;
+			return last;
 		}
+		last = last->nextMember;
 	}
-	return last;
+	return 0;
+}
+
+ILMember *ILClassNextMemberMatch(ILClass *info, ILMember *last, int kind,
+								 const char *name, ILType *signature)
+{
+	info = ILClassGetUnderlying(info);
+	if(last)
+	{
+		last = last->nextMember;
+	}
+	else if(info)
+	{
+		last = info->firstMember;
+	}
+	else
+	{
+		return 0;
+	}
+	while(last != 0)
+	{
+		if(kind && ((int)(last->kind)) != kind)
+		{
+			last = last->nextMember;
+			continue;
+		}
+		if(name && strcmp(last->name, name) != 0)
+		{
+			last = last->nextMember;
+			continue;
+		}
+		if(!signature || ILTypeIdentical(last->signature, signature))
+		{
+			return last;
+		}
+		last = last->nextMember;
+	}
+	return 0;
 }
 
 void ILClassMarkComplete(ILClass *info)
@@ -1202,70 +1417,71 @@ ILType *ILClassToType(ILClass *info)
 	}
 
 	/* Check for system classes with primitive equivalents */
-	if(info->namespace && !strcmp(info->namespace, "System") &&
+	if(info->className->namespace &&
+	   !strcmp(info->className->namespace, "System") &&
 	   ILClassGetNestedParent(info) == 0)
 	{
-		if(!strcmp(info->name, "Boolean"))
+		if(!strcmp(info->className->name, "Boolean"))
 		{
 			return ILType_Boolean;
 		}
-		else if(!strcmp(info->name, "SByte"))
+		else if(!strcmp(info->className->name, "SByte"))
 		{
 			return ILType_Int8;
 		}
-		else if(!strcmp(info->name, "Byte"))
+		else if(!strcmp(info->className->name, "Byte"))
 		{
 			return ILType_UInt8;
 		}
-		else if(!strcmp(info->name, "Int16"))
+		else if(!strcmp(info->className->name, "Int16"))
 		{
 			return ILType_Int16;
 		}
-		else if(!strcmp(info->name, "UInt16"))
+		else if(!strcmp(info->className->name, "UInt16"))
 		{
 			return ILType_UInt16;
 		}
-		else if(!strcmp(info->name, "Char"))
+		else if(!strcmp(info->className->name, "Char"))
 		{
 			return ILType_Char;
 		}
-		else if(!strcmp(info->name, "Int32"))
+		else if(!strcmp(info->className->name, "Int32"))
 		{
 			return ILType_Int32;
 		}
-		else if(!strcmp(info->name, "UInt32"))
+		else if(!strcmp(info->className->name, "UInt32"))
 		{
 			return ILType_UInt32;
 		}
-		else if(!strcmp(info->name, "Int64"))
+		else if(!strcmp(info->className->name, "Int64"))
 		{
 			return ILType_Int64;
 		}
-		else if(!strcmp(info->name, "UInt64"))
+		else if(!strcmp(info->className->name, "UInt64"))
 		{
 			return ILType_UInt64;
 		}
-		else if(!strcmp(info->name, "Single"))
+		else if(!strcmp(info->className->name, "Single"))
 		{
 			return ILType_Float32;
 		}
-		else if(!strcmp(info->name, "Double"))
+		else if(!strcmp(info->className->name, "Double"))
 		{
 			return ILType_Float64;
 		}
-		else if(!strcmp(info->name, "IntPtr"))
+		else if(!strcmp(info->className->name, "IntPtr"))
 		{
 			return ILType_Int;
 		}
-		else if(!strcmp(info->name, "UIntPtr"))
+		else if(!strcmp(info->className->name, "UIntPtr"))
 		{
 			return ILType_UInt;
 		}
-		else if(!strcmp(info->name, "Void"))
+		else if(!strcmp(info->className->name, "Void"))
 		{
 			return ILType_Void;
 		}
-		else if(!strcmp(info->name, "TypedReference"))
+		else if(!strcmp(info->className->name, "TypedReference"))
 		{
 			return ILType_TypedRef;
 		}
@@ -1604,6 +1820,36 @@ done:
 	return result;
 }
 
+ILMethod *ILClassGetMethodImplForProxy(ILClass *info, ILMethod *method)
+{
+	ILClass *parent;
+	ILMethod *method2;
+	ILMethod *result;
+	const char *name;
+	ILType *signature;
+
+	/* Cache the name and signature information for the interface method */
+	name = ILMethod_Name(method);
+	signature = ILMethod_Signature(method);
+
+	/* Start at the parent class and search for any public method that
+	   matches the specified signature */
+	parent = ILClassGetParent(info);
+	result = 0;
+	while(parent != 0)
+	{
+		method2 = (ILMethod *)ILClassNextMemberMatch
+			(parent, 0, IL_META_MEMBERKIND_METHOD, name, signature);
+		if(method2 && ILMethod_IsPublic(method2))
+		{
+			result = method2;
+			break;
+		}
+		parent = ILClassGetParent(parent);
+	}
+	return result;
+}
+
 void ILClassDetachMember(ILMember *member)
 {
 	ILClass *info = member->owner;
@@ -1646,6 +1892,19 @@ void ILClassAttachMember(ILClass *info, ILMember *member)
 	info->lastMember = member;
 }
 
+int ILClassNamespaceIsValid(ILContext *context, const char *nspace)
+{
+	if(nspace)
+	{
+		return (ILHashFind(context->namespaceHash, nspace) != 0);
+	}
+	else
+	{
+		/* The empty namespace name is always valid */
+		return 1;
+	}
+}
+
 #ifdef	__cplusplus
 };
 #endif
