@@ -34,49 +34,61 @@ extern	"C" {
 			"/usr/local/lib:/usr/X11R6/lib:/usr/lib"
 
 /*
- * The cached system path, loaded from the environment.
+ * The cached system path, loaded from the environment.  The "important"
+ * system path is searched before looking in the same directory as the
+ * assembly, to prevent conflicts with foreign CLI's for core libraries.
  */
 static char **systemPath = 0;
 static int systemPathSize = 0;
-
-/*
- * The explicit list of pnetlib assemblies which must override
- * any others with these names that we may encounter (e.g. Mono's).
- * We detect the pnetlib versions by looking for "pnetlib.here".
- */
-static const char * const systemAssemblies[] = {
-	"mscorlib",
-	"System",
-	"System.Xml",
-};
-#define	systemAssembliesSize	\
-			(sizeof(systemAssemblies) / sizeof(systemAssemblies[0]))
+static char **importantSystemPath = 0;
+static int importantSystemPathSize = 0;
 
 /*
  * Add a pathname to "systemPaths".
  */
-static void AddSystemPath(const char *path, int len)
+static void AddSystemPath(const char *path, int len, int importantPath)
 {
 	char **newPathList;
-	newPathList = (char **)ILRealloc
-		(systemPath, (systemPathSize + 1) * sizeof(char *));
-	if(!newPathList)
+	if(importantPath)
 	{
-		return;
+		newPathList = (char **)ILRealloc
+			(importantSystemPath,
+			 (importantSystemPathSize + 1) * sizeof(char *));
+		if(!newPathList)
+		{
+			return;
+		}
+		importantSystemPath = newPathList;
+		importantSystemPath[importantSystemPathSize] = ILDupNString(path, len);
+		if(!(importantSystemPath[importantSystemPathSize]))
+		{
+			return;
+		}
+		++importantSystemPathSize;
 	}
-	systemPath = newPathList;
-	systemPath[systemPathSize] = ILDupNString(path, len);
-	if(!(systemPath[systemPathSize]))
+	else
 	{
-		return;
+		newPathList = (char **)ILRealloc
+			(systemPath, (systemPathSize + 1) * sizeof(char *));
+		if(!newPathList)
+		{
+			return;
+		}
+		systemPath = newPathList;
+		systemPath[systemPathSize] = ILDupNString(path, len);
+		if(!(systemPath[systemPathSize]))
+		{
+			return;
+		}
+		++systemPathSize;
 	}
-	++systemPathSize;
 }
 
 /*
  * Split a pathname list and add it to the global list of system paths.
  */
-static void SplitPathString(char *list, char *stdpath, char *defaultPath)
+static void SplitPathString(char *list, char *stdpath, char *defaultPath,
+							int importantPath)
 {
 	int len;
 	int separator;
@@ -87,7 +99,7 @@ static void SplitPathString(char *list, char *stdpath, char *defaultPath)
 		/* If we have a standard path, then add that */
 		if(stdpath)
 		{
-			AddSystemPath(stdpath, strlen(stdpath));
+			AddSystemPath(stdpath, strlen(stdpath), importantPath);
 			ILFree(stdpath);
 			stdpath = 0;
 		}
@@ -162,7 +174,7 @@ static void SplitPathString(char *list, char *stdpath, char *defaultPath)
 		}
 
 		/* Add the path to the global list */
-		AddSystemPath(list, len);
+		AddSystemPath(list, len, importantPath);
 
 		/* Advance to the next path */
 		list += len;
@@ -175,16 +187,18 @@ static void SplitPathString(char *list, char *stdpath, char *defaultPath)
 static void LoadSystemPath(void)
 {
 #if !defined(__palmos__)
-	if(!systemPath)
+	if(!systemPath && !importantSystemPath)
 	{
 		SplitPathString(getenv("CSCC_LIB_PATH"),
 						ILGetStandardLibraryPath("cscc/lib"),
-						CSCC_LIB_PATH_DEFAULT);
-		SplitPathString(getenv("MONO_PATH"), ILGetStandardLibraryPath(0), 0);
-		SplitPathString(getenv("LD_LIBRARY_PATH"), 0, LD_LIBRARY_PATH_DEFAULT);
+						CSCC_LIB_PATH_DEFAULT, 1);
+		SplitPathString(getenv("MONO_PATH"),
+						ILGetStandardLibraryPath(0), 0, 0);
+		SplitPathString(getenv("LD_LIBRARY_PATH"), 0,
+						LD_LIBRARY_PATH_DEFAULT, 0);
 	#ifdef IL_WIN32_PLATFORM
 		/* Win32: try looking in PATH also, since Win32 puts dll's there */
-		SplitPathString(getenv("PATH"), 0, 0);
+		SplitPathString(getenv("PATH"), 0, 0, 0);
 	#endif
 	}
 #endif
@@ -374,32 +388,42 @@ static int TestPathForPnetlibHere(const char *pathname, int pathlen)
  */
 static int FindAssemblyInPaths(const char **paths, unsigned long numPaths,
 							   const char *name, int namelen, int isSystem,
-							   char **path, char **firstPath)
+							   char **path, char **firstPath,
+							   const ILUInt16 *version)
 {
 	unsigned long posn;
+	char versionBuffer[64];
+	const char *versionPrefix;
+	if(version && (version[0] != 0 || version[1] != 0 ||
+				   version[2] != 0 || version[3] != 0))
+	{
+		sprintf(versionBuffer, "%d.%d.%d.%d/",
+				(int)(version[0]), (int)(version[1]),
+				(int)(version[2]), (int)(version[3]));
+		versionPrefix = versionBuffer;
+	}
+	else
+	{
+		versionBuffer[0] = '\0';
+		versionPrefix = 0;
+	}
 	for(posn = 0; posn < numPaths; ++posn)
 	{
 		*path = TestPathForFile(paths[posn], strlen(paths[posn]),
-							    name, namelen, 0, ".dll", 0, 0);
+							    name, namelen, versionPrefix, ".dll", 0, 0);
 		if(*path)
 		{
-			if(!isSystem ||
-			   TestPathForPnetlibHere(paths[posn], strlen(paths[posn])))
+			return 1;
+		}
+		if(versionPrefix)
+		{
+			/* Retry without the version directory prefix */
+			*path = TestPathForFile(paths[posn], strlen(paths[posn]),
+								    name, namelen, 0, ".dll", 0, 0);
+			if(*path)
 			{
-				if(*firstPath)
-				{
-					ILFree(*firstPath);
-				}
 				return 1;
 			}
-			if(*firstPath)
-			{
-				ILFree(*path);
-			}
-			else
-			{
-				*firstPath = *path;
-			}
 		}
 	}
 	return 0;
@@ -412,7 +436,6 @@ char *ILImageSearchPath(const char *name, const ILUInt16 *version,
 						int suppressStandardPaths, int *sameDir)
 {
 	int namelen;
-	unsigned long posn;
 	char *path;
 	char *firstPath;
 	int len;
@@ -442,19 +465,37 @@ char *ILImageSearchPath(const char *name, const ILUInt16 *version,
 	}
 
 	/* Is this a system assembly that we must handle specially? */
+	/* Note: system check is no longer necessary - remove eventually */
 	isSystem = 0;
-	for(posn = 0; posn < systemAssembliesSize; ++posn)
+
+	/* Initialize flag variables for the search */
+	if(sameDir)
 	{
-		if(namelen == strlen(systemAssemblies[posn]) &&
-		   !strncmp(systemAssemblies[posn], name, namelen))
+		*sameDir = 0;
+	}
+	firstPath = 0;
+
+	/* Search the before path list */
+	if(FindAssemblyInPaths(beforePaths, numBeforePaths, name, namelen,
+						   isSystem, &path, &firstPath, version))
+	{
+		return path;
+	}
+
+	/* Search the "important" standard system paths */
+	if(!suppressStandardPaths)
+	{
+		LoadSystemPath();
+		if(FindAssemblyInPaths((const char **)importantSystemPath,
+							   (unsigned long)importantSystemPathSize,
+							   name, namelen, isSystem, &path, &firstPath,
+							   version))
 		{
-			isSystem = 1;
-			break;
+			return path;
 		}
 	}
 
 	/* Look in the same directory as the parent assembly */
-	firstPath = 0;
 	if(sameDir)
 	{
 		*sameDir = 1;
@@ -487,20 +528,13 @@ char *ILImageSearchPath(const char *name, const ILUInt16 *version,
 		*sameDir = 0;
 	}
 
-	/* Search the before path list */
-	if(FindAssemblyInPaths(beforePaths, numBeforePaths, name, namelen,
-						   isSystem, &path, &firstPath))
-	{
-		return path;
-	}
-
 	/* Search the standard system paths */
 	if(!suppressStandardPaths)
 	{
 		LoadSystemPath();
 		if(FindAssemblyInPaths((const char **)systemPath,
 							   (unsigned long)systemPathSize, name, namelen,
-							   isSystem, &path, &firstPath))
+							   isSystem, &path, &firstPath, version))
 		{
 			return path;
 		}
@@ -508,7 +542,7 @@ char *ILImageSearchPath(const char *name, const ILUInt16 *version,
 
 	/* Search the after path list */
 	if(FindAssemblyInPaths(afterPaths, numAfterPaths, name, namelen,
-						   isSystem, &path, &firstPath))
+						   isSystem, &path, &firstPath, version))
 	{
 		return path;
 	}
@@ -528,14 +562,17 @@ int _ILImageDynamicLink(ILImage *image, const char *filename, int flags)
 	int sameDir;
 	int loadFlags;
 	int len, retryLower;
+	ILImage *linkImage;
 
 	/* Scan the AssemblyRef table for the assemblies that we require */
 	assem = 0;
 	while((assem = (ILAssembly *)ILImageNextToken
 				(image, IL_META_TOKEN_ASSEMBLY_REF, assem)) != 0)
 	{
-		/* Ignore this assembly reference if we already have it */
-		if(ILContextGetAssembly(image->context, assem->name) != 0)
+		/* Ignore this assembly reference if we already have it and
+		   it isn't marked as "building" */
+		linkImage = ILContextGetAssembly(image->context, assem->name);
+		if(linkImage && linkImage->type != IL_IMAGETYPE_BUILDING)
 		{
 			continue;
 		}
@@ -668,6 +705,82 @@ int _ILImageDynamicLink(ILImage *image, const char *filename, int flags)
 	return 0;
 }
 
+int _ILImageDynamicLinkModule(ILImage *image, const char *filename,
+							  const char *moduleName, int flags,
+							  ILImage **newImage)
+{
+	char *pathname;
+	int error;
+	int len, retryLower;
+
+	/* Clear the return image before we start */
+	*newImage = 0;
+
+	/* If we loaded the parent from an insecure source, then bail out
+	   without attempting to load the module files */
+	if((flags & IL_LOADFLAG_INSECURE) != 0 || !filename)
+	{
+		return 0;
+	}
+
+	/* Strip the final component from the filename */
+	len = strlen(filename);
+	while(len > 0 && filename[len - 1] != '/' && filename[len - 1] != '\\')
+	{
+		--len;
+	}
+	if(len > 0)
+	{
+		--len;
+	}
+
+	/* Ignore this module if we already have it */
+	if(ILContextGetFile(image->context, moduleName) != 0)
+	{
+		return 0;
+	}
+
+	/* Ignore this module if its name contains a '/' or '\', because
+	   files in other directories may be a security risk */
+	if(ILMemChr(moduleName, '/', strlen(moduleName)) != 0 ||
+	   ILMemChr(moduleName, '\\', strlen(moduleName)) != 0)
+	{
+		return 0;
+	}
+
+	/* Get the full pathname of the referenced file */
+	retryLower = 0;
+	pathname = TestPathForFile(filename, len,
+							   moduleName, strlen(moduleName),
+							   0, 0, &retryLower, 0);
+	if(!pathname && retryLower)
+	{
+		pathname = TestPathForFile(filename, len,
+								   moduleName, strlen(moduleName),
+								   0, 0, &retryLower, 1);
+	}
+	if(!pathname)
+	{
+	#if IL_DEBUG_META
+		fprintf(stderr, "could not locate the file %s\n", moduleName);
+	#endif
+		return IL_LOADERR_UNRESOLVED;
+	}
+
+	/* Load the image */
+	error = ILImageLoadFromFile(pathname, image->context,
+						        newImage, flags, 0);
+	ILFree(pathname);
+	if(error == -1)
+	{
+		return IL_LOADERR_UNRESOLVED;
+	}
+	else
+	{
+		return error;
+	}
+}
+
 int ILImageLoadAssembly(const char *name, ILContext *context,
 						ILImage *parentImage, ILImage **image)
 {
@@ -972,14 +1085,15 @@ char *ILPInvokeResolveModule(ILPInvoke *pinvoke)
 		name = remapName;
 	}
 
-	/* If the name already includes a directory specification,
+	/* If the name already includes a root directory specification,
 	   then assume that this is the path we are looking for */
-	for(posn = 0; posn < namelen; ++posn)
+	if(namelen > 0 && (name[0] == '/' || name[0] == '\\'))
 	{
-		if(name[posn] == '/' || name[posn] == '\\')
-		{
-			return ILDupNString(name, namelen);
-		}
+		return ILDupNString(name, namelen);
+	}
+	else if(namelen > 1 && name[1] == ':')
+	{
+		return ILDupNString(name, namelen);
 	}
 
 	/* Determine the platform-specific prefix and suffix to add */
@@ -1081,6 +1195,18 @@ char *ILPInvokeResolveModule(ILPInvoke *pinvoke)
 
 	/* Look on the standard system search path */
 	LoadSystemPath();
+	for(posn = 0; posn < importantSystemPathSize; ++posn)
+	{
+		fullName = TestPathForNativeLib(importantSystemPath[posn],
+									    strlen(importantSystemPath[posn]),
+									    baseName, strlen(baseName),
+										optPrefix, suffix);
+		if(fullName)
+		{
+			ILFree(baseName);
+			return fullName;
+		}
+	}
 	for(posn = 0; posn < systemPathSize; ++posn)
 	{
 		fullName = TestPathForNativeLib(systemPath[posn],
