@@ -798,6 +798,45 @@ const void *ILFileDeclGetHash(ILFileDecl *decl, unsigned long *len)
 	return ILImageGetBlob(decl->programItem.image, decl->hash, len);
 }
 
+ILImage *ILFileDeclToImage(ILFileDecl *decl)
+{
+	if(decl->programItem.linked)
+	{
+		/* We have cached a previous image */
+		return _ILProgramItemResolve(&(decl->programItem))->image;
+	}
+	else
+	{
+		/* Search for a matching ILModule in another image */
+		ILImage *image;
+		ILModule *newModule = 0;
+		image = decl->programItem.image->context->firstImage;
+		while(image != 0 && newModule == 0)
+		{
+			newModule = 0;
+			while((newModule = (ILModule *)ILImageNextToken
+						(image, IL_META_TOKEN_MODULE, newModule)) != 0)
+			{
+				if(!ILStrICmp(newModule->name, decl->name))
+				{
+					break;
+				}
+			}
+			image = image->nextImage;
+		}
+		if(!newModule)
+		{
+			return 0;
+		}
+
+		/* Link the reference to the foreign module */
+		_ILProgramItemLink(&(decl->programItem), &(newModule->programItem));
+
+		/* Return the new module's image to the caller */
+		return newModule->programItem.image;
+	}
+}
+
 ILManifestRes *ILManifestResCreate(ILImage *image, ILToken token,
 								   const char *name, ILUInt32 attrs)
 {
@@ -884,7 +923,8 @@ ILUInt32 ILManifestResGetAttrs(ILManifestRes *res)
 ILExportedType *ILExportedTypeCreate(ILImage *image, ILToken token,
 									 ILUInt32 attributes,
 									 const char *name,
-									 const char *namespace)
+									 const char *namespace,
+									 ILProgramItem *scope)
 {
 	ILExportedType *type;
 
@@ -899,11 +939,11 @@ ILExportedType *ILExportedTypeCreate(ILImage *image, ILToken token,
 	type->classItem.programItem.image = image;
 	type->classItem.attributes = attributes | IL_META_TYPEDEF_REFERENCE;
 	type->classItem.className =
-		_ILClassNameLookup(image, 0, 0, name, namespace);
+		_ILClassNameLookup(image, scope, 0, name, namespace);
 	if(!(type->classItem.className))
 	{
 		type->classItem.className =
-			_ILClassNameCreate(image, token, name, namespace, 0, 0);
+			_ILClassNameCreate(image, token, name, namespace, scope, 0);
 	}
 	if(!(type->classItem.className))
 	{
