@@ -85,6 +85,11 @@ ILProgramItem *_ILProgramItemResolve(ILProgramItem *item);
  */
 ILProgramItem *_ILProgramItemResolveRef(ILProgramItem *item);
 
+/*
+ * Load the attributes for a program item on demand.
+ */
+void _ILProgramItemLoadAttributes(ILProgramItem *item);
+
 /*
  * Information about a custom attribute.
  */
@@ -218,6 +223,45 @@ struct _tagJavaConstEntry
 
 };
 
+/*
+ * Information about a class name.
+ */
+struct _tagILClassName
+{
+	ILImage		   *image;			/* Image containing the class */
+	ILToken			token;			/* Token code for the class */
+	const char	   *name;			/* Name of the class */
+	const char	   *namespace;		/* Name of the class's namespace */
+	ILProgramItem  *scope;			/* Scope that the class is defined in */
+	ILClassName	   *scopeName;		/* Name of the scope (nested classes) */
+};
+
+/*
+ * Create a new class name.
+ */
+ILClassName *_ILClassNameCreate(ILImage *image, ILToken token,
+								const char *name, const char *namespace,
+								ILProgramItem *scopeItem,
+								ILClassName *scopeName);
+
+/*
+ * Update class name token information for a class.
+ */
+void _ILClassNameUpdateToken(ILClass *info);
+
+/*
+ * Convert a class name into the actual class, performing dynamic
+ * token lookup as necessary.
+ */
+ILClass *_ILClassNameToClass(ILClassName *className);
+
+/*
+ * Look up a class name within a specific scope.
+ */
+ILClassName *_ILClassNameLookup(ILImage *image, ILProgramItem *scopeItem,
+								ILClassName *scopeName, const char *name,
+								const char *namespace);
+
 /*
  * Information about a class.
  */
@@ -225,14 +269,12 @@ struct _tagILClass
 {
 	ILProgramItem	programItem;		/* Parent class fields */
 	ILUInt32		attributes;			/* IL_META_TYPEDEF_xxx flags */
-	const char	   *name;				/* Name of the class */
-	const char	   *namespace;			/* Name of the class's namespace */
+	ILClassName    *className;			/* Name information for the class */
 	ILClass		   *parent;				/* Parent class */
 	ILImplements   *implements;			/* List of implemented interfaces */
 	ILMember	   *firstMember;		/* First member owned by the class */
 	ILMember	   *lastMember;			/* Last member owned by the class */
 	ILNestedInfo   *nestedChildren;		/* List of nested children */
-	ILProgramItem  *scope;				/* Scope that the class is defined in */
 	ILType         *synthetic;			/* Synthetic type for this class */
 	ILClassExt     *ext;				/* Extension information */
 	void           *userData;			/* Data added by the runtime engine */
@@ -245,7 +287,9 @@ struct _tagILClass
 #define	IL_META_TYPEDEF_REFERENCE		0x80000000	/* Not yet really defined */
 #define	IL_META_TYPEDEF_COMPLETE		0x40000000	/* Definition is complete */
 #define	IL_META_TYPEDEF_CCTOR_ONCE		0x20000000	/* .cctor already done */
-#define	IL_META_TYPEDEF_SYSTEM_MASK		0xE0000000	/* System flags */
+#define	IL_META_TYPEDEF_GENERIC_PARS	0x10000000	/* Has generic parameters */
+#define	IL_META_TYPEDEF_CLASS_EXPANDED	0x08000000	/* Generics expanded */
+#define	IL_META_TYPEDEF_SYSTEM_MASK		0xF8000000	/* System flags */
 
 /*
  * Information about an "implements" clause for a class.
@@ -311,6 +355,11 @@ struct _tagILMethod
 
 };
 
+/*
+ * Load the parameter definitions for a method token on demand.
+ */
+void _ILMethodLoadParams(ILMethod *method);
+
 /*
  * Information about a parameter.
  */
@@ -352,6 +401,8 @@ struct _tagILProperty
 {
 	ILMember		member;				/* Common member fields */
 	ILMethodSem    *semantics;			/* List of semantic methods */
+	ILMethod	   *getter;				/* Cached copy of the getter */
+	ILMethod	   *setter;				/* Cached copy of the setter */
 
 };
 
@@ -362,7 +413,7 @@ struct _tagILProperty
 struct _tagILPInvoke
 {
 	ILMember		member;				/* Common member fields */
-	ILMethod	   *method;				/* Method PInvoke applies to */
+	ILMember	   *memberInfo;			/* Member PInvoke applies to */
 	ILModule	   *module;				/* Module function is imported from */
 	const char	   *aliasName;			/* Alias for the function */
 
@@ -608,15 +659,42 @@ struct _tagILManifestRes
  */
 struct _tagILExportedType
 {
-	ILProgramItem	programItem;		/* Parent class fields */
-	ILUInt32		attributes;			/* Attributes for the type */
+	ILClass			classItem;			/* Parent class fields */
 	ILUInt32		identifier;			/* Foreign identifier for the type */
-	const char     *name;				/* Name for the type */
-	const char     *namespace;			/* Namespace for the type */
-	ILProgramItem  *scope;				/* Scope where the type exists */
 
 };
 
+/*
+ * Information about a generic parameter.
+ */
+struct _tagILGenericPar
+{
+	ILOwnedItem		ownedItem;			/* Parent class fields */
+	ILUInt16		number;				/* Parameter number */
+	ILUInt16		flags;				/* Parameter flags */
+	const char     *name;				/* Parameter name */
+	ILProgramItem  *kind;				/* Parameter kind */
+	ILProgramItem  *constraint;			/* Parameter constraint */
+
+};
+
+/*
+ * Information about a generic method specification.
+ */
+struct _tagILMethodSpec
+{
+	ILProgramItem	programItem;		/* Parent class fields */
+	ILMember       *method;				/* Method specification applies to */
+	ILType         *type;				/* Instantiation information */
+	ILUInt32		typeBlob;			/* Blob offset of the type */
+
+};
+
+/*
+ * Set the value of a MethodSpec type to an explicit blob index.
+ */
+void _ILMethodSpecSetTypeIndex(ILMethodSpec *spec, ILUInt32 index);
+
 #ifdef	__cplusplus
 };
 #endif
