@@ -2138,6 +2138,12 @@ int ILFileDeclSetHash(ILFileDecl *decl, const void *hash, unsigned long len);
  */
 const void *ILFileDeclGetHash(ILFileDecl *decl, unsigned long *len);
 
+/*
+ * Convert a file declaration into a fully resolved image.
+ * Returns NULL if insufficient linkages available.
+ */
+ILImage *ILFileDeclToImage(ILFileDecl *decl);
+
 /*
  * Helper macros for querying information about file declarations.
  */
@@ -2223,7 +2229,8 @@ ILUInt32 ILManifestResGetAttrs(ILManifestRes *res);
 ILExportedType *ILExportedTypeCreate(ILImage *image, ILToken token,
 									 ILUInt32 attributes,
 									 const char *name,
-									 const char *nspace);
+									 const char *nspace,
+									 ILProgramItem *scope);
 
 /*
  * Set the foreign type ID associated with an exported type declaration.
