@@ -37,7 +37,7 @@
 #ifdef HAVE_FCNTL_H
         #include <fcntl.h>
 #endif
-#ifdef _WIN32
+#ifdef IL_WIN32_PLATFORM
 	#include <windows.h>
 	#include <io.h>
 #endif
@@ -48,6 +48,7 @@
 	#include <utime.h>
 #endif
 #endif
+#include <errno.h>
 
 #ifdef	__cplusplus
 extern	"C" {
@@ -62,11 +63,17 @@ extern	"C" {
 int ILDeleteFile(const char *filename)
 {
 #ifdef HAVE_UNLINK
-	return unlink(filename);
+	if(unlink(filename) == 0)
+		return IL_ERRNO_Success;
+	else
+		return ILSysIOConvertErrno(errno);
+#elif defined(HAVE_REMOVE)
+	if(remove(filename) == 0)
+		return IL_ERRNO_Success;
+	else
+		return ILSysIOConvertErrno(errno);
 #else
-#ifdef HAVE_REMOVE
-	return remove(filename);
-#endif
+	return IL_ERRNO_ENOSYS;
 #endif
 }
 
@@ -353,37 +360,51 @@ int ILSysIOFlushWrite(ILSysIOHandle handle)
 
 ILInt32 ILCopyFile(const char *src, const char *dest)
 {
-	ILSysIOHandle src_result;
-	ILSysIOHandle dest_result;
-	char buf[4000];
-	int bytes_read;
+	FILE *infile;
+	FILE *outfile;
+	char buffer[BUFSIZ];
+	int len;
 
-	/* Reset errno because we're not sure what it'll be */
-	errno = 0;
-	if (src == NULL || dest == NULL)
+	/* Bail out if either of the arguments is invalid */
+	if(!src || !dest)
 	{
 	    return IL_ERRNO_ENOENT;
 	}
 
-	/* Open the source and dest files */
-	src_result = ILSysIOOpenFile(src, ILFileMode_Open, ILFileAccess_Read, ILFileShare_Read);
-	dest_result = ILSysIOOpenFile(dest, ILFileMode_CreateNew, ILFileAccess_Write, ILFileShare_None);
-
-	/* if we had an error, send the errno back */
-	if(src_result == ILSysIOHandle_Invalid || dest_result == ILSysIOHandle_Invalid)
+	/* Open the input file */
+	if((infile = fopen(src, "rb")) == NULL)
 	{
-		return ILSysIOConvertErrno(errno);
+		if((infile = fopen(src, "r")) == NULL)
+		{
+			return ILSysIOConvertErrno(errno);
+		}
 	}
 
-	while ( ( bytes_read = ILSysIORead(src_result, buf, 4000) ) != 0 && errno == 0)
+	/* Open the output file */
+	if((outfile = fopen(dest, "wb")) == NULL)
 	{
-		ILSysIOWrite(dest_result, buf, bytes_read);
+		if((outfile = fopen(dest, "w")) == NULL)
+		{
+			int error = ILSysIOConvertErrno(errno);
+			fclose(infile);
+			return error;
+		}
 	}
 
-	ILSysIOClose(src_result);
-	ILSysIOClose(dest_result);
-	
-	return ILSysIOConvertErrno(errno);
+	/* Copy the file contents */
+	while((len = (int)fread(buffer, 1, sizeof(buffer), infile)) > 0)
+	{
+		fwrite(buffer, 1, len, outfile);
+		if(len < sizeof(buffer))
+		{
+			break;
+		}
+	}
+
+	/* Close the files and exit */
+	fclose(infile);
+	fclose(outfile);
+	return 0;
 }
 
 int ILSysIOTruncate(ILSysIOHandle handle, ILInt64 posn)
@@ -407,14 +428,94 @@ int ILSysIOTruncate(ILSysIOHandle handle, ILInt64 posn)
 
 int ILSysIOLock(ILSysIOHandle handle, ILInt64 position, ILInt64 length)
 {
-	/* Locking is not yet supported - TODO */
+#if defined(IL_WIN32_PLATFORM)
+	/* Bypass the system library and call LockFile directly under Win32 */
+#ifdef IL_WIN32_CYGWIN
+	HANDLE osHandle = (HANDLE)get_osfhandle((int)(ILNativeInt)handle);
+#else
+	HANDLE osHandle = (HANDLE)_get_osfhandle((int)(ILNativeInt)handle);
+#endif
+	if(osHandle == (HANDLE)INVALID_HANDLE_VALUE)
+	{
+		ILSysIOSetErrno(IL_ERRNO_EBADF);
+		return 0;
+	}
+	if(LockFile(osHandle,
+				(DWORD)(position & IL_MAX_UINT32),
+			 	(DWORD)((position >> 32) & IL_MAX_UINT32),
+			 	(DWORD)(length & IL_MAX_UINT32),
+			 	(DWORD)((length >> 32) & IL_MAX_UINT32)))
+	{
+		return 1;
+	}
+	else
+	{
+		ILSysIOSetErrno(IL_ERRNO_ENOLCK);
+		return 0;
+	}
+#elif defined(HAVE_FCNTL) && defined(HAVE_F_SETLKW)
+	struct flock cntl_data;
+	/* set fields individually...who knows what extras are there? */
+	cntl_data.l_type = F_WRLCK;
+	cntl_data.l_whence = SEEK_SET;
+	/* actually, off_t changes in LFS on 32bit, so be careful */
+	cntl_data.l_start = (off_t)(ILNativeInt) position;
+	cntl_data.l_len = (off_t)(ILNativeInt) length;
+	/* -1 is error, anything else is OK */
+	if (fcntl ((int)(ILNativeInt) handle, F_SETLKW, &cntl_data) != -1)
+		return 1;
+	else
+		return 0;
+#else /* !defined(HAVE_FCNTL) || !defined(HAVE_F_SETLKW) */
+	/* Locking is not supported w/o fcntl - TODO */
 	return 1;
+#endif /* !defined(HAVE_FCNTL) || !defined(HAVE_F_SETLKW) */
 }
 
 int ILSysIOUnlock(ILSysIOHandle handle, ILInt64 position, ILInt64 length)
 {
-	/* Locking is not yet supported - TODO */
+#if defined(IL_WIN32_PLATFORM)
+	/* Bypass the system library and call UnlockFile directly under Win32 */
+#ifdef IL_WIN32_CYGWIN
+	HANDLE osHandle = (HANDLE)get_osfhandle((int)(ILNativeInt)handle);
+#else
+	HANDLE osHandle = (HANDLE)_get_osfhandle((int)(ILNativeInt)handle);
+#endif
+	if(osHandle == (HANDLE)INVALID_HANDLE_VALUE)
+	{
+		ILSysIOSetErrno(IL_ERRNO_EBADF);
+		return 0;
+	}
+	if(UnlockFile(osHandle,
+				  (DWORD)(position & IL_MAX_UINT32),
+			 	  (DWORD)((position >> 32) & IL_MAX_UINT32),
+			 	  (DWORD)(length & IL_MAX_UINT32),
+			 	  (DWORD)((length >> 32) & IL_MAX_UINT32)))
+	{
+		return 1;
+	}
+	else
+	{
+		ILSysIOSetErrno(IL_ERRNO_ENOLCK);
+		return 0;
+	}
+#elif defined(HAVE_FCNTL) && defined(HAVE_F_SETLKW)
+	struct flock cntl_data;
+	/* set fields individually...who knows what extras are there? */
+	cntl_data.l_type = F_UNLCK;
+	cntl_data.l_whence = SEEK_SET;
+	/* actually, off_t changes in LFS on 32bit, so be careful */
+	cntl_data.l_start = (off_t)(ILNativeInt) position;
+	cntl_data.l_len = (off_t)(ILNativeInt) length;
+	/* -1 is error, anything else is OK */
+	if (fcntl ((int)(ILNativeInt) handle, F_SETLKW, &cntl_data) != -1)
+		return 1;
+	else
+		return 0;
+#else /* !defined(HAVE_FCNTL) || !defined(HAVE_F_SETLKW) */
+	/* Unlocking is not supported w/o fcntl - TODO */
 	return 1;
+#endif /* !defined(HAVE_FCNTL) || !defined(HAVE_F_SETLKW) */
 }
 
 int ILSysIOHasAsync(void)
